@@ -26,9 +26,7 @@
 #ifndef EPHY_EMBED_SHELL_H
 #define EPHY_EMBED_SHELL_H
 
-#include <gtk/gtk.h>
-
-#include "ephy-download.h"
+#include <webkit2/webkit2.h>
 
 G_BEGIN_DECLS
 
@@ -50,7 +48,8 @@ typedef enum
   EPHY_EMBED_SHELL_MODE_PRIVATE,
   EPHY_EMBED_SHELL_MODE_INCOGNITO,
   EPHY_EMBED_SHELL_MODE_APPLICATION,
-  EPHY_EMBED_SHELL_MODE_TEST
+  EPHY_EMBED_SHELL_MODE_TEST,
+  EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER
 } EphyEmbedShellMode;
 
 #define EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE(mode) \
@@ -68,9 +67,6 @@ struct _EphyEmbedShellClass
 {
   GtkApplicationClass parent_class;
 
-  void    (* download_added)   (EphyEmbedShell *shell, EphyDownload *download);
-  void    (* download_removed) (EphyEmbedShell *shell, EphyDownload *download);
-
   void    (* prepare_close)    (EphyEmbedShell *shell);
 
   void    (* restored_window)  (EphyEmbedShell *shell);
@@ -78,6 +74,7 @@ struct _EphyEmbedShellClass
 
 GType              ephy_embed_shell_get_type                   (void);
 EphyEmbedShell    *ephy_embed_shell_get_default                (void);
+WebKitWebContext  *ephy_embed_shell_get_web_context            (EphyEmbedShell   *shell);
 GObject           *ephy_embed_shell_get_global_history_service (EphyEmbedShell   *shell);
 GObject           *ephy_embed_shell_get_encodings              (EphyEmbedShell   *shell);
 void               ephy_embed_shell_prepare_close              (EphyEmbedShell   *shell);
@@ -88,18 +85,17 @@ GtkPageSetup      *ephy_embed_shell_get_page_setup             (EphyEmbedShell
 void               ephy_embed_shell_set_print_settings         (EphyEmbedShell   *shell,
                                                                 GtkPrintSettings *settings);
 GtkPrintSettings  *ephy_embed_shell_get_print_settings         (EphyEmbedShell   *shell);
-GList             *ephy_embed_shell_get_downloads              (EphyEmbedShell   *shell);
-void               ephy_embed_shell_add_download               (EphyEmbedShell   *shell,
-                                                                EphyDownload     *download);
-void               ephy_embed_shell_remove_download            (EphyEmbedShell   *shell,
-                                                                EphyDownload     *download);
 EphyEmbedShellMode ephy_embed_shell_get_mode                   (EphyEmbedShell   *shell);
 gboolean           ephy_embed_shell_launch_handler             (EphyEmbedShell   *shell,
                                                                 GFile            *file,
                                                                 const char       *mime_type,
                                                                 guint32           user_time);
-void               ephy_embed_shell_clear_cache                (EphyEmbedShell *shell);
-GDBusProxy        *ephy_embed_shell_get_web_extension_proxy    (EphyEmbedShell   *shell);
+void               ephy_embed_shell_clear_cache                (EphyEmbedShell   *shell);
+void               ephy_embed_shell_set_thumbanil_path         (EphyEmbedShell   *shell,
+                                                                const char       *url,
+                                                                time_t            mtime,
+                                                                const char       *path);
+WebKitUserContentManager *ephy_embed_shell_get_user_content_manager (EphyEmbedShell *shell);
 
 G_END_DECLS
 
