@@ -42,6 +42,7 @@
 #include "ephy-settings.h"
 #include "ephy-snapshot-service.h"
 #include "ephy-string.h"
+#include "ephy-uri-helpers.h"
 #include "ephy-web-app-utils.h"
 #include "ephy-web-dom-utils.h"
 #include "ephy-web-extension-proxy.h"
@@ -78,6 +79,7 @@ struct _EphyWebViewPrivate {
   guint ever_committed : 1;
 
   char *address;
+  char *display_address;
   char *typed_address;
   char *loading_message;
   char *link_message;
@@ -489,9 +491,9 @@ ephy_web_view_create_form_auth_save_confirmation_info_bar (EphyWebView *web_view
 
   LOG ("Going to show infobar about %s", webkit_web_view_get_uri (WEBKIT_WEB_VIEW (web_view)));
 
-  info_bar = gtk_info_bar_new_with_buttons (_("Save"), GTK_RESPONSE_YES,
+  info_bar = gtk_info_bar_new_with_buttons (_("_Don’t Save"), GTK_RESPONSE_CLOSE,
+                                            _("_Save"), GTK_RESPONSE_YES,
                                             NULL);
-  gtk_info_bar_set_show_close_button (GTK_INFO_BAR (info_bar), TRUE);
 
   action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
   gtk_orientable_set_orientation (GTK_ORIENTABLE (action_area),
@@ -644,11 +646,6 @@ icon_changed_cb (EphyWebView *view,
                  GParamSpec *pspec,
                  gpointer user_data)
 {
-  if (view->priv->snapshot_timeout_id == 0) {
-    view->priv->snapshot_timeout_id = g_timeout_add_full (G_PRIORITY_LOW, 0,
-                                                          (GSourceFunc)web_view_check_snapshot,
-                                                          view, NULL);
-  }
   _ephy_web_view_update_icon (view);
 }
 
@@ -674,7 +671,7 @@ form_auth_data_save_confirmation_response (GtkInfoBar *info_bar,
 }
 
 static void
-form_auth_data_save_requested (EphyWebExtensionProxy *web_extension,
+form_auth_data_save_requested (EphyEmbedShell *shell,
                                guint request_id,
                                guint64 page_id,
                                const char *hostname,
@@ -699,7 +696,7 @@ form_auth_data_save_requested (EphyWebExtensionProxy *web_extension,
 }
 
 static void
-allow_tls_certificate_cb (EphyWebExtensionProxy *shell,
+allow_tls_certificate_cb (EphyEmbedShell *shell,
                           guint64 page_id,
                           EphyWebView *web_view)
 {
@@ -713,7 +710,7 @@ allow_tls_certificate_cb (EphyWebExtensionProxy *shell,
   g_return_if_fail (priv->tls_error_failing_uri != NULL);
 
   uri = soup_uri_new (priv->tls_error_failing_uri);
-  webkit_web_context_allow_tls_certificate_for_host (webkit_web_context_get_default (),
+  webkit_web_context_allow_tls_certificate_for_host (ephy_embed_shell_get_web_context (shell),
                                                      priv->certificate,
                                                      uri->host);
   ephy_web_view_load_url (web_view, ephy_web_view_get_address (web_view));
@@ -734,11 +731,11 @@ page_created_cb (EphyEmbedShell *shell,
   priv->web_extension = web_extension;
   g_object_add_weak_pointer (G_OBJECT (priv->web_extension), (gpointer *)&priv->web_extension);
 
-  g_signal_connect_object (priv->web_extension, "form-auth-data-save-requested",
+  g_signal_connect_object (shell, "form-auth-data-save-requested",
                            G_CALLBACK (form_auth_data_save_requested),
                            web_view, 0);
 
-  g_signal_connect_object (priv->web_extension, "allow-tls-certificate",
+  g_signal_connect_object (shell, "allow-tls-certificate",
                            G_CALLBACK (allow_tls_certificate_cb),
                            web_view, 0);
 }
@@ -748,6 +745,18 @@ ephy_web_view_dispose (GObject *object)
 {
   EphyWebViewPrivate *priv = EPHY_WEB_VIEW (object)->priv;
 
+  if (priv->web_extension)
+    {
+      g_object_remove_weak_pointer (G_OBJECT (priv->web_extension), (gpointer *)&priv->web_extension);
+      priv->web_extension = NULL;
+    }
+
+  if (priv->password_info_bar)
+    {
+      g_object_remove_weak_pointer (G_OBJECT (priv->password_info_bar), (gpointer *)&priv->password_info_bar);
+      priv->password_info_bar = NULL;
+    }
+
   g_signal_handlers_disconnect_by_func (priv->history_service,
                                         ephy_web_view_history_cleared_cb,
                                         EPHY_WEB_VIEW (object));
@@ -779,6 +788,7 @@ ephy_web_view_finalize (GObject *object)
   ephy_web_view_popups_manager_reset (EPHY_WEB_VIEW (object));
 
   g_free (priv->address);
+  g_free (priv->display_address);
   g_free (priv->typed_address);
   g_free (priv->link_message);
   g_free (priv->loading_message);
@@ -816,7 +826,7 @@ title_changed_cb (WebKitWebView *web_view,
   if (!title && uri)
     title = title_from_address = ephy_embed_utils_get_title_from_address (uri);
 
-  if (uri && title && !ephy_web_view_is_history_frozen (webview))
+  if (uri && title && *title && !ephy_web_view_is_history_frozen (webview))
     ephy_history_service_set_url_title (history, uri, title, NULL, NULL, NULL);
 
   g_free (title_from_address);
@@ -842,6 +852,9 @@ ephy_web_view_set_address (EphyWebView *view,
   g_free (priv->address);
   priv->address = g_strdup (address);
 
+  g_free (priv->display_address);
+  priv->display_address = ephy_uri_safe_unescape (priv->address);
+
   is_blank = address == NULL ||
              strcmp (address, "about:blank") == 0;
   _ephy_web_view_set_is_blank (view, is_blank);
@@ -949,7 +962,7 @@ ephy_web_view_class_init (EphyWebViewClass *klass)
 /**
  * EphyWebView:address:
  *
- * View's current address.
+ * View's current address. This is a percent-encoded URI.
  **/
   g_object_class_install_property (gobject_class,
                                    PROP_ADDRESS,
@@ -1379,9 +1392,9 @@ decide_policy_cb (WebKitWebView *web_view,
 }
 
 static void
-decide_on_geolocation_policy_request (GtkWidget *info_bar,
-                                      int response,
-                                      WebKitPermissionRequest *request)
+decide_on_permission_request (GtkWidget *info_bar,
+                              int response,
+                              WebKitPermissionRequest *request)
 {
   gtk_widget_destroy (info_bar);
 
@@ -1409,22 +1422,38 @@ permission_request_cb (WebKitWebView           *web_view,
   char *message;
   char *host;
 
-  if (!WEBKIT_IS_GEOLOCATION_PERMISSION_REQUEST (decision))
+  if (!WEBKIT_IS_GEOLOCATION_PERMISSION_REQUEST (decision) &&
+      !WEBKIT_IS_NOTIFICATION_PERMISSION_REQUEST (decision))
     return FALSE;
 
+  /* Application mode implies being OK with notifications. */
+  if (WEBKIT_IS_NOTIFICATION_PERMISSION_REQUEST (decision) &&
+      ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) == EPHY_EMBED_SHELL_MODE_APPLICATION) {
+    webkit_permission_request_allow (decision);
+    return TRUE;
+  }
+
   info_bar = gtk_info_bar_new_with_buttons (_("Deny"), GTK_RESPONSE_NO,
                                             _("Allow"), GTK_RESPONSE_YES,
                                             NULL);
 
   action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
-  /* Translators: Geolocation policy for a specific site. */
   gtk_orientable_set_orientation (GTK_ORIENTABLE (action_area),
                                   GTK_ORIENTATION_HORIZONTAL);
 
   /* Label */
   host = ephy_string_get_host_name (webkit_web_view_get_uri (web_view));
-  message = g_markup_printf_escaped (_("The page at <b>%s</b> wants to know your location."),
-                                     host);
+
+  if (WEBKIT_IS_GEOLOCATION_PERMISSION_REQUEST (decision)) {
+    /* Translators: Geolocation policy for a specific site. */
+    message = g_markup_printf_escaped (_("The page at <b>%s</b> wants to know your location."),
+                                       host);
+  } else {
+    /* Translators: Notification policy for a specific site. */
+    message = g_markup_printf_escaped (_("The page at <b>%s</b> wants to show desktop notifications."),
+                                       host);
+  }
+
   g_free (host);
 
   label = gtk_label_new (NULL);
@@ -1440,7 +1469,7 @@ permission_request_cb (WebKitWebView           *web_view,
 
   /* Ref the decision, to keep it alive while we decide */
   g_signal_connect (info_bar, "response",
-                    G_CALLBACK (decide_on_geolocation_policy_request),
+                    G_CALLBACK (decide_on_permission_request),
                     g_object_ref (decision));
 
   ephy_embed_add_top_widget (EPHY_GET_EMBED_FROM_EPHY_WEB_VIEW (web_view),
@@ -1501,9 +1530,11 @@ ephy_web_view_set_loading_message (EphyWebView *view,
 
   g_clear_pointer (&priv->loading_message, g_free);
   if (address) {
+    char *decoded_address;
     char *title;
 
-    title = ephy_embed_utils_get_title_from_address (address);
+    decoded_address = ephy_uri_safe_unescape (address);
+    title = ephy_embed_utils_get_title_from_address (decoded_address);
 
     if (title != NULL && title[0] != '\0') {
       /* translators: %s here is the address of the web page */
@@ -1512,6 +1543,7 @@ ephy_web_view_set_loading_message (EphyWebView *view,
       priv->loading_message = g_strdup (_("Loading…"));
     }
 
+    g_free (decoded_address);
     g_free (title);
   }
 
@@ -1654,8 +1686,8 @@ load_changed_cb (WebKitWebView *web_view,
     if (!ephy_web_view_is_history_frozen (view) &&
         ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) != EPHY_EMBED_SHELL_MODE_INCOGNITO) {
       if (!ephy_snapshot_service_lookup_snapshot_path (ephy_snapshot_service_get_default (), webkit_web_view_get_uri (web_view))) {
-        /* If the snapshot check hasn't been scheduled already by the favicon callback,
-         * check the snapshot if we don't get a favicon in 1 second
+        /* FIXME: The 1s delay is a workaround to allow time to render the page and get a favicon.
+         * https://bugzilla.gnome.org/show_bug.cgi?id=761065
          */
         if (priv->snapshot_timeout_id == 0) {
           priv->snapshot_timeout_id = g_timeout_add_seconds_full (G_PRIORITY_LOW, 1,
@@ -1712,7 +1744,7 @@ get_style_sheet (void)
 {
   const gchar *file;
   GError *error = NULL;
-  char *sheet;
+  char *sheet = NULL;
 
   file = ephy_file ("error.css");
   if (file && !g_file_get_contents (file, &sheet, NULL, &error)) {
@@ -1731,37 +1763,37 @@ detailed_message_from_tls_errors (GTlsCertificateFlags tls_errors)
 
   if (tls_errors & G_TLS_CERTIFICATE_BAD_IDENTITY) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site presented identification that belongs to a different web site."));
+    g_ptr_array_add (errors, _("This website presented identification that belongs to a different website."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_EXPIRED) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification is too old to trust. Check the date on your computer’s calendar."));
+    g_ptr_array_add (errors, _("This website’s identification is too old to trust. Check the date on your computer’s calendar."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_UNKNOWN_CA) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification was not issued by a trusted organization."));
+    g_ptr_array_add (errors, _("This website’s identification was not issued by a trusted organization."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_GENERIC_ERROR) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification could not be processed. It may be corrupted."));
+    g_ptr_array_add (errors, _("This website’s identification could not be processed. It may be corrupted."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_REVOKED) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification has been revoked by the trusted organization that issued it."));
+    g_ptr_array_add (errors, _("This website’s identification has been revoked by the trusted organization that issued it."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_INSECURE) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification cannot be trusted because it uses very weak encryption."));
+    g_ptr_array_add (errors, _("This website’s identification cannot be trusted because it uses very weak encryption."));
   }
 
   if (tls_errors & G_TLS_CERTIFICATE_NOT_ACTIVATED) {
     /* Possible error message when a site presents a bad certificate. */
-    g_ptr_array_add (errors, _("This web site’s identification is only valid for future dates. Check the date on your computer’s calendar."));
+    g_ptr_array_add (errors, _("This website’s identification is only valid for future dates. Check the date on your computer’s calendar."));
   }
 
   if (errors->len == 1) {
@@ -1802,15 +1834,15 @@ get_tls_error_page_message (EphyWebView *view, const char *hostname)
   msg = g_strdup_printf ("<p>%s</p><p>%s</p><p>%s</p><p>%s <strong>%s</strong></p>",
                          warning,
                          /* Message when a site's TLS certificate is invalid. */
-                         _("When you try to connect securely, web sites present "
+                         _("When you try to connect securely, websites present "
                            "identification to prove that your connection has not been "
                            "maliciously intercepted. There is something wrong with "
-                           "this site’s identification:"),
+                           "this website’s identification:"),
                          details,
                          /* Message when a site's TLS certificate is invalid. */
                          _("A third party may have hijacked your connection. You should "
                            "continue only if you know there is a good reason why this "
-                           "site does not use trusted identification."),
+                           "website does not use trusted identification."),
                          /* Good advice from Firefox; displays when a site's TLS certificate is invalid. */
                          _("Legitimate banks, stores, and other public sites will "
                            "not ask you to do this."));
@@ -1908,7 +1940,8 @@ ephy_web_view_load_error_page (EphyWebView *view,
       /* Button on error page when a website's TLS certificate is invalid. */
       button_label = g_strdup (_("Load Anyway"));
       custom_class = "tls-error";
-      load_anyway_js = g_strdup ("EpiphanyTLSCertificateErrorPage.allowException();");
+      load_anyway_js = g_strdup_printf ("window.webkit.messageHandlers.tlsErrorPage.postMessage(%"G_GUINT64_FORMAT");",
+                                        webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (view)));
       break;
     default:
       return;
@@ -1961,7 +1994,8 @@ load_failed_cb (WebKitWebView *web_view,
   priv->load_failed = TRUE;
   ephy_web_view_set_link_message (view, NULL);
 
-  if (error->domain == SOUP_HTTP_ERROR) {
+  if (error->domain == SOUP_HTTP_ERROR ||
+      error->domain == G_TLS_ERROR) {
     ephy_web_view_load_error_page (view, uri, EPHY_WEB_VIEW_ERROR_PAGE_NETWORK_ERROR, error);
     return TRUE;
   }
@@ -2179,8 +2213,11 @@ ephy_web_view_init (EphyWebView *web_view)
 GtkWidget *
 ephy_web_view_new (void)
 {
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
+
   return g_object_new (EPHY_TYPE_WEB_VIEW,
-                       "user-content-manager", ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ()),
+                       "web-context", ephy_embed_shell_get_web_context (shell),
+                       "user-content-manager", ephy_embed_shell_get_user_content_manager (shell),
                        "settings", ephy_embed_prefs_get_settings (),
                        NULL);
 }
@@ -2321,7 +2358,9 @@ ephy_web_view_is_overview (EphyWebView *view)
  * ephy_web_view_get_address:
  * @view: an #EphyWebView
  *
- * Returns the address of the currently loaded page.
+ * Returns the address of the currently-loaded page, percent-encoded.
+ * This URI should not be displayed to the user; to do that, use
+ * ephy_web_view_get_display_address().
  *
  * Return value: @view's address. Will never be %NULL.
  **/
@@ -2332,6 +2371,24 @@ ephy_web_view_get_address (EphyWebView *view)
   return priv->address ? priv->address : "about:blank";
 }
 
+/**
+ * ephy_web_view_get_display_address:
+ * @view: an #EphyWebView
+ *
+ * Returns the display address of the currently-loaded page. This is a
+ * decoded URI suitable for display to the user. To get a URI suitable
+ * for sending to a server, e.g. for storage in the bookmarks or history
+ * database, use ephy_web_view_get_address().
+ *
+ * Return value: @view's address. Will never be %NULL.
+ */
+const char *
+ephy_web_view_get_display_address (EphyWebView *view)
+{
+  EphyWebViewPrivate *priv = view->priv;
+  return priv->display_address ? priv->display_address : "about:blank";
+}
+
 /**
  * ephy_web_view_is_loading:
  * @view: an #EphyWebView
@@ -2460,16 +2517,17 @@ ephy_web_view_get_link_message (EphyWebView *view)
 /**
  * ephy_web_view_set_link_message:
  * @view: an #EphyWebView
- * @link_message: new value for link-message in @view
+ * @address: new value for link-message in @view
  *
  * Sets the value of link-message property which tells the URL of the hovered
  * link.
  **/
 void
 ephy_web_view_set_link_message (EphyWebView *view,
-                                const char *link_message)
+                                const char *address)
 {
   EphyWebViewPrivate *priv;
+  char *decoded_address;
 
   g_return_if_fail (EPHY_IS_WEB_VIEW (view));
 
@@ -2477,7 +2535,13 @@ ephy_web_view_set_link_message (EphyWebView *view,
 
   g_free (priv->link_message);
 
-  priv->link_message = ephy_embed_utils_link_message_parse (link_message);
+  if (address) {
+    decoded_address = ephy_uri_safe_unescape (address);
+    priv->link_message = ephy_embed_utils_link_message_parse (decoded_address);
+    g_free (decoded_address);
+  } else {
+    priv->link_message = NULL;
+  }
 
   g_object_notify (G_OBJECT (view), "status-message");
   g_object_notify (G_OBJECT (view), "link-message");
