@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim: set sw=2 ts=2 sts=2 et: */
 /*
- *  Copyright © 2012 Igalia S.L.
+ *  Copyright © 2014 Igalia S.L.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,26 +23,46 @@
 
 #include "ephy-debug.h"
 #include "ephy-embed-form-auth.h"
+#include "ephy-file-helpers.h"
 #include "ephy-form-auth-data.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
 #include "ephy-web-dom-utils.h"
+#include "ephy-uri-helpers.h"
 #include "uri-tester.h"
+#include "ephy-web-overview.h"
+#include "ephy-web-extension-names.h"
 
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <libsoup/soup.h>
+#include <string.h>
 #include <webkit2/webkit-web-extension.h>
+#define WEBKIT_DOM_USE_UNSTABLE_API
+#include <webkitdom/WebKitDOMDOMWindowUnstable.h>
+#include <JavaScriptCore/JavaScript.h>
 
+struct _EphyWebExtensionPrivate
+{
+  WebKitWebExtension *extension;
+  gboolean initialized;
 
-/* FIXME: These global variables should be freed somehow. */
-static UriTester *uri_tester;
-static EphyFormAuthDataCache *form_auth_data_cache;
-static GDBusConnection *dbus_connection;
+  GDBusConnection *dbus_connection;
+  guint registration_id;
+  GArray *page_created_signals_pending;
+
+  UriTester *uri_tester;
+  EphyFormAuthDataCache *form_auth_data_cache;
+  GHashTable *form_auth_data_save_requests;
+  EphyWebOverviewModel *overview_model;
+};
 
 static const char introspection_xml[] =
   "<node>"
   " <interface name='org.gnome.Epiphany.WebExtension'>"
+  "  <signal name='PageCreated'>"
+  "   <arg type='t' name='page_id' direction='out'/>"
+  "  </signal>"
   "  <method name='HasModifiedForms'>"
   "   <arg type='t' name='page_id' direction='in'/>"
   "   <arg type='b' name='has_modified_forms' direction='out'/>"
@@ -58,31 +78,48 @@ static const char introspection_xml[] =
   "   <arg type='s' name='uri' direction='out'/>"
   "   <arg type='s' name='color' direction='out'/>"
   "  </method>"
-  "  <signal name='FormAuthDataSaveConfirmationRequired'>"
-  "   <arg type='u' name='request_id' direction='out'/>"
-  "   <arg type='t' name='page_id' direction='out'/>"
-  "   <arg type='s' name='hostname' direction='out'/>"
-  "   <arg type='s' name='username' direction='out'/>"
-  "  </signal>"
   "  <method name='FormAuthDataSaveConfirmationResponse'>"
   "   <arg type='u' name='request_id' direction='in'/>"
   "   <arg type='b' name='should_store' direction='in'/>"
   "  </method>"
+  "  <method name='HistorySetURLs'>"
+  "   <arg type='a(ss)' name='urls' direction='in'/>"
+  "  </method>"
+  "  <method name='HistorySetURLThumbnail'>"
+  "   <arg type='s' name='url' direction='in'/>"
+  "   <arg type='s' name='path' direction='in'/>"
+  "  </method>"
+  "  <method name='HistorySetURLTitle'>"
+  "   <arg type='s' name='url' direction='in'/>"
+  "   <arg type='s' name='title' direction='in'/>"
+  "  </method>"
+  "  <method name='HistoryDeleteURL'>"
+  "   <arg type='s' name='url' direction='in'/>"
+  "  </method>"
+  "  <method name='HistoryDeleteHost'>"
+  "   <arg type='s' name='host' direction='in'/>"
+  "  </method>"
+  "  <method name='HistoryClear'/>"
   " </interface>"
   "</node>";
 
+G_DEFINE_TYPE (EphyWebExtension, ephy_web_extension, G_TYPE_OBJECT)
 
 static gboolean
 web_page_send_request (WebKitWebPage *web_page,
                        WebKitURIRequest *request,
                        WebKitURIResponse *redirected_response,
-                       gpointer user_data)
+                       EphyWebExtension *extension)
 {
   const char *request_uri;
   const char *page_uri;
+  gboolean ret;
+
+  request_uri = webkit_uri_request_get_uri (request);
 
   if (g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DO_NOT_TRACK)) {
     SoupMessageHeaders *headers;
+    char *new_uri;
 
     headers = webkit_uri_request_get_http_headers (request);
     if (headers) {
@@ -90,12 +127,19 @@ web_page_send_request (WebKitWebPage *web_page,
        * http://tools.ietf.org/id/draft-mayer-do-not-track-00.txt */
       soup_message_headers_append (headers, "DNT", "1");
     }
+
+    /* Remove analytics from URL before loading */
+    new_uri = ephy_remove_tracking_from_uri (request_uri);
+    if (new_uri) {
+      webkit_uri_request_set_uri (request, new_uri);
+      request_uri = webkit_uri_request_get_uri (request);
+    }
+    g_free (new_uri);
   }
 
   if (!g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ENABLE_ADBLOCK))
       return FALSE;
 
-  request_uri = webkit_uri_request_get_uri (request);
   page_uri = webkit_web_page_get_uri (web_page);
 
   /* Always load the main resource. */
@@ -106,23 +150,25 @@ web_page_send_request (WebKitWebPage *web_page,
   if (g_str_has_prefix (request_uri, SOUP_URI_SCHEME_DATA))
       return FALSE;
 
-  return uri_tester_test_uri (uri_tester, request_uri, page_uri, AD_URI_CHECK_TYPE_OTHER);
+  ret = uri_tester_test_uri (extension->priv->uri_tester, request_uri, page_uri, AD_URI_CHECK_TYPE_OTHER);
+  if (ret)
+    g_debug ("Request '%s' blocked (page: '%s')", request_uri, page_uri);
+
+  return ret;
 }
 
 static GHashTable *
-get_form_auth_data_save_requests (void)
+ephy_web_extension_get_form_auth_data_save_requests (EphyWebExtension *extension)
 {
-  static GHashTable *form_auth_data_save_requests = NULL;
-
-  if (!form_auth_data_save_requests) {
-    form_auth_data_save_requests =
+  if (!extension->priv->form_auth_data_save_requests) {
+    extension->priv->form_auth_data_save_requests =
       g_hash_table_new_full (g_direct_hash,
                              g_direct_equal,
                              NULL,
                              (GDestroyNotify)g_object_unref);
   }
 
-  return form_auth_data_save_requests;
+  return extension->priv->form_auth_data_save_requests;
 }
 
 static guint
@@ -142,11 +188,15 @@ store_password (EphyEmbedFormAuth *form_auth)
   char *username_field_value = NULL;
   char *password_field_name = NULL;
   char *password_field_value = NULL;
+  WebKitDOMNode *username_node;
+  EphyWebExtension *extension = ephy_web_extension_get ();
 
-  g_object_get (ephy_embed_form_auth_get_username_node (form_auth),
-                "name", &username_field_name,
-                "value", &username_field_value,
-                NULL);
+  username_node = ephy_embed_form_auth_get_username_node (form_auth);
+  if (username_node)
+    g_object_get (username_node,
+                  "name", &username_field_name,
+                  "value", &username_field_value,
+                  NULL);
   g_object_get (ephy_embed_form_auth_get_password_node (form_auth),
                 "name", &password_field_name,
                 "value", &password_field_value,
@@ -163,7 +213,7 @@ store_password (EphyEmbedFormAuth *form_auth)
   g_free (uri_str);
 
   /* Update internal caching */
-  ephy_form_auth_data_cache_add (form_auth_data_cache,
+  ephy_form_auth_data_cache_add (extension->priv->form_auth_data_cache,
                                  uri->host,
                                  username_field_name,
                                  password_field_name,
@@ -181,39 +231,42 @@ request_decision_on_storing (EphyEmbedFormAuth *form_auth)
   char *username_field_value = NULL;
   guint request_id;
   SoupURI *uri;
-  GError *error = NULL;
+  WebKitDOMNode *username_node;
+  WebKitDOMDOMWindow *dom_window;
+  GVariant *variant;
+  gchar *message;
 
-  if (!dbus_connection) {
+  dom_window = webkit_dom_document_get_default_view (ephy_embed_form_auth_get_owner_document (form_auth));
+  if (!dom_window) {
     g_object_unref (form_auth);
     return;
   }
 
   request_id = form_auth_data_save_request_new_id ();
   uri = ephy_embed_form_auth_get_uri (form_auth);
-  g_object_get (ephy_embed_form_auth_get_username_node (form_auth),
-                "value", &username_field_value, NULL);
+  username_node = ephy_embed_form_auth_get_username_node (form_auth);
+  if (username_node)
+    g_object_get (username_node, "value", &username_field_value, NULL);
 
-  g_dbus_connection_emit_signal (dbus_connection,
-                                 NULL,
-                                 EPHY_WEB_EXTENSION_OBJECT_PATH,
-                                 EPHY_WEB_EXTENSION_INTERFACE,
-                                 "FormAuthDataSaveConfirmationRequired",
-                                 g_variant_new ("(utss)",
-                                                request_id,
-                                                ephy_embed_form_auth_get_page_id (form_auth),
-                                                uri ? uri->host : "",
-                                                username_field_value ? username_field_value : ""),
-                                 &error);
-  if (error) {
-    g_warning ("Error emitting signal FormAuthDataSaveConfirmationRequired: %s\n", error->message);
-    g_error_free (error);
-  } else {
-    g_hash_table_insert (get_form_auth_data_save_requests (),
+  variant = g_variant_new ("(utss)",
+                           request_id,
+                           ephy_embed_form_auth_get_page_id (form_auth),
+                           uri ? uri->host : "",
+                           username_field_value ? username_field_value : "");
+  g_free (username_field_value);
+
+  message = g_variant_print (variant, FALSE);
+  g_variant_unref (variant);
+
+  if (webkit_dom_dom_window_webkit_message_handlers_post_message (dom_window, "formAuthData", message)) {
+    g_hash_table_insert (ephy_web_extension_get_form_auth_data_save_requests (ephy_web_extension_get ()),
                          GINT_TO_POINTER (request_id),
                          g_object_ref (form_auth));
+  } else {
+    g_warning ("Error sending formAuthData message");
   }
 
-  g_free (username_field_value);
+  g_free (message);
   g_object_unref (form_auth);
 }
 
@@ -224,18 +277,20 @@ should_store_cb (const char *username,
 {
   EphyEmbedFormAuth *form_auth = EPHY_EMBED_FORM_AUTH (user_data);
 
-  if (username && password) {
+  if (password) {
+    WebKitDOMNode *username_node;
     char *username_field_value = NULL;
     char *password_field_value = NULL;
 
-    g_object_get (ephy_embed_form_auth_get_username_node (form_auth),
-                  "value", &username_field_value, NULL);
+    username_node = ephy_embed_form_auth_get_username_node (form_auth);
+    if (username_node)
+      g_object_get (username_node, "value", &username_field_value, NULL);
     g_object_get (ephy_embed_form_auth_get_password_node (form_auth),
                   "value", &password_field_value, NULL);
 
     /* FIXME: We use only the first result, for now; We need to do
      * something smarter here */
-    if (g_str_equal (username, username_field_value) &&
+    if (g_strcmp0 (username, username_field_value) == 0 &&
         g_str_equal (password, password_field_value)) {
       LOG ("User/password already stored. Not asking about storing.");
     } else {
@@ -268,16 +323,19 @@ form_submitted_cb (WebKitDOMHTMLFormElement *dom_form,
   if (!ephy_web_dom_utils_find_form_auth_elements (dom_form, &username_node, &password_node))
     return TRUE;
 
-  g_object_get (username_node,
-                "value", &username_field_value,
-                NULL);
+  if (username_node) {
+    g_object_get (username_node,
+                  "value", &username_field_value,
+                  NULL);
+  }
 
   /* EphyEmbedFormAuth takes ownership of the nodes */
   form_auth = ephy_embed_form_auth_new (web_page, username_node, password_node, username_field_value);
   uri = ephy_embed_form_auth_get_uri (form_auth);
   soup_uri_set_query (uri, NULL);
 
-  g_object_get (username_node, "name", &username_field_name, NULL);
+  if (username_node)
+    g_object_get (username_node, "name", &username_field_name, NULL);
   g_object_get (password_node, "name", &password_field_name, NULL);
   uri_str = soup_uri_to_string (uri, FALSE);
 
@@ -303,15 +361,18 @@ fill_form_cb (const char *username,
               gpointer user_data)
 {
   EphyEmbedFormAuth *form_auth = EPHY_EMBED_FORM_AUTH (user_data);
+  WebKitDOMNode *username_node;
 
   if (username == NULL && password == NULL) {
     LOG ("No result");
     return;
   }
 
-  LOG ("Found: user %s pass (hidden)", username);
-  g_object_set (ephy_embed_form_auth_get_username_node (form_auth),
-                "value", username, NULL);
+  username_node = ephy_embed_form_auth_get_username_node (form_auth);
+
+  LOG ("Found: user %s pass (hidden)", username_node ? username : "(none)");
+  if (username_node)
+    g_object_set (username_node, "value", username, NULL);
   g_object_set (ephy_embed_form_auth_get_password_node (form_auth),
                 "value", password, NULL);
 }
@@ -320,12 +381,14 @@ static gint
 ephy_form_auth_data_compare (EphyFormAuthData *form_data,
                              EphyEmbedFormAuth *form_auth)
 {
-  char *username_field_name;
+  WebKitDOMNode *username_node;
+  char *username_field_name = NULL;
   char *password_field_name;
   gboolean retval;
 
-  g_object_get (ephy_embed_form_auth_get_username_node (form_auth),
-                "name", &username_field_name, NULL);
+  username_node = ephy_embed_form_auth_get_username_node (form_auth);
+  if (username_node)
+    g_object_get (username_node, "name", &username_field_name, NULL);
   g_object_get (ephy_embed_form_auth_get_password_node (form_auth),
                 "name", &password_field_name, NULL);
 
@@ -347,12 +410,15 @@ pre_fill_form (EphyEmbedFormAuth *form_auth)
   SoupURI *uri;
   char *uri_str;
   char *username;
+  WebKitDOMNode *username_node;
+  EphyWebExtension *extension;
 
   uri = ephy_embed_form_auth_get_uri (form_auth);
   if (!uri)
     return;
 
-  form_auth_data_list = ephy_form_auth_data_cache_get_list (form_auth_data_cache, uri->host);
+  extension = ephy_web_extension_get ();
+  form_auth_data_list = ephy_form_auth_data_cache_get_list (extension->priv->form_auth_data_cache, uri->host);
   l = g_slist_find_custom (form_auth_data_list, form_auth, (GCompareFunc)ephy_form_auth_data_compare);
   if (!l)
     return;
@@ -360,12 +426,14 @@ pre_fill_form (EphyEmbedFormAuth *form_auth)
   form_data = (EphyFormAuthData *)l->data;
   uri_str = soup_uri_to_string (uri, FALSE);
 
-  g_object_get (ephy_embed_form_auth_get_username_node (form_auth),
-                "value", &username,
-                NULL);
+  username_node = ephy_embed_form_auth_get_username_node (form_auth);
+  if (username_node)
+    g_object_get (username_node, "value", &username, NULL);
+  else
+    username = NULL;
 
   /* The username node is empty, so pre-fill with the default. */
-  if (g_str_equal (username, ""))
+  if (username != NULL && g_str_equal (username, ""))
     g_clear_pointer (&username, g_free);
 
   ephy_form_auth_data_query (uri_str,
@@ -425,7 +493,7 @@ user_chosen_cb (WebKitDOMNode  *li,
 
 GtkStyleContext *global_entry_context = NULL;
 static GtkStyleContext*
-get_entry_style_context ()
+get_entry_style_context (void)
 {
   GtkWidgetPath *path;
 
@@ -444,9 +512,11 @@ get_entry_style_context ()
 }
 
 static char*
-get_selected_bgcolor ()
+get_selected_bgcolor (void)
 {
   GdkRGBA color;
+  gtk_style_context_set_state (get_entry_style_context (),
+                               GTK_STATE_FLAG_SELECTED);
   gtk_style_context_get_background_color (get_entry_style_context (),
                                           GTK_STATE_FLAG_SELECTED,
                                           &color);
@@ -454,9 +524,11 @@ get_selected_bgcolor ()
 }
 
 static char*
-get_selected_fgcolor ()
+get_selected_fgcolor (void)
 {
   GdkRGBA color;
+  gtk_style_context_set_state (get_entry_style_context (),
+                               GTK_STATE_FLAG_SELECTED);
   gtk_style_context_get_color (get_entry_style_context (),
                                GTK_STATE_FLAG_SELECTED,
                                &color);
@@ -464,9 +536,11 @@ get_selected_fgcolor ()
 }
 
 static char*
-get_bgcolor ()
+get_bgcolor (void)
 {
   GdkRGBA color;
+  gtk_style_context_set_state (get_entry_style_context (),
+                               GTK_STATE_FLAG_NORMAL);
   gtk_style_context_get_background_color (get_entry_style_context (),
                                           GTK_STATE_FLAG_NORMAL,
                                           &color);
@@ -474,9 +548,11 @@ get_bgcolor ()
 }
 
 static char*
-get_fgcolor ()
+get_fgcolor (void)
 {
   GdkRGBA color;
+  gtk_style_context_set_state (get_entry_style_context (),
+                               GTK_STATE_FLAG_NORMAL);
   gtk_style_context_get_color (get_entry_style_context (),
                                GTK_STATE_FLAG_NORMAL,
                                &color);
@@ -522,7 +598,7 @@ get_user_choice_anchor_style (gboolean selected)
   return style_attribute;
 }
 
-void
+static void
 show_user_choices (WebKitDOMDocument *document,
                    WebKitDOMNode     *username_node)
 {
@@ -532,16 +608,17 @@ show_user_choices (WebKitDOMDocument *document,
   GSList *iter;
   GSList *auth_data_list;
   gboolean username_node_ever_edited;
-  long x, y;
-  long input_width;
+  double x, y;
+  double input_width;
   char *style_attribute;
-  const char* username;
+  char* username;
 
   g_object_get (username_node,
                 "value", &username,
-                "offset-width", &input_width,
                 NULL);
 
+  input_width = webkit_dom_element_get_offset_width (WEBKIT_DOM_ELEMENT (username_node));
+
   main_div = webkit_dom_document_create_element (document, "div", NULL);
   webkit_dom_element_set_attribute (main_div, "id", "ephy-user-choices-container", NULL);
 
@@ -552,13 +629,13 @@ show_user_choices (WebKitDOMDocument *document,
    */
   style_attribute = g_strdup_printf ("position: absolute; z-index: 2147483647;"
                                      "cursor: default;"
-                                     "width: %ldpx;"
+                                     "width: %lfpx;"
                                      "background-color: white;"
                                      "box-shadow: 5px 5px 5px black;"
                                      "border-top: 0;"
                                      "border-radius: 8px;"
                                      "-webkit-user-modify: read-only ! important;"
-                                     "left: %ldpx; top: %ldpx;",
+                                     "left: %lfpx; top: %lfpx;",
                                      input_width, x, y);
 
   webkit_dom_element_set_attribute (main_div, "style", style_attribute, NULL);
@@ -634,6 +711,7 @@ show_user_choices (WebKitDOMDocument *document,
                                       NULL);
   }
 
+  g_free  (username);
   body = WEBKIT_DOM_NODE (webkit_dom_document_get_body (document));
   webkit_dom_node_append_child (WEBKIT_DOM_NODE (body),
                                 WEBKIT_DOM_NODE (main_div),
@@ -826,14 +904,14 @@ form_destroyed_cb (gpointer form_auth, GObject *form)
 
 static void
 web_page_document_loaded (WebKitWebPage *web_page,
-                          gpointer user_data)
+                          EphyWebExtension *extension)
 {
   WebKitDOMHTMLCollection *forms = NULL;
   WebKitDOMDocument *document = NULL;
   gulong forms_n;
   int i;
 
-  if (!form_auth_data_cache ||
+  if (!extension->priv->form_auth_data_cache ||
       !g_settings_get_boolean (EPHY_SETTINGS_MAIN, EPHY_PREFS_REMEMBER_PASSWORDS))
     return;
 
@@ -868,19 +946,21 @@ web_page_document_loaded (WebKitWebPage *web_page,
       webkit_dom_event_target_add_event_listener (WEBKIT_DOM_EVENT_TARGET (form), "submit",
                                                   G_CALLBACK (form_submitted_cb), FALSE,
                                                   web_page);
-      webkit_dom_event_target_add_event_listener (WEBKIT_DOM_EVENT_TARGET (username_node), "blur",
-                                                  G_CALLBACK (username_changed_cb), FALSE,
-                                                  form_auth);
+      if (username_node) {
+        webkit_dom_event_target_add_event_listener (WEBKIT_DOM_EVENT_TARGET (username_node), "blur",
+                                                    G_CALLBACK (username_changed_cb), FALSE,
+                                                    form_auth);
+      }
 
       /* Plug in the user autocomplete */
       uri_string = webkit_web_page_get_uri (web_page);
       uri = soup_uri_new (uri_string);
 
-      auth_data_list = ephy_form_auth_data_cache_get_list (form_auth_data_cache, uri->host);
+      auth_data_list = ephy_form_auth_data_cache_get_list (extension->priv->form_auth_data_cache, uri->host);
 
       soup_uri_free (uri);
 
-      if (auth_data_list && auth_data_list->next) {
+      if (auth_data_list && auth_data_list->next && username_node) {
         LOG ("More than 1 password saved, hooking menu for choosing which on focus");
         g_object_set_data (G_OBJECT (username_node), "ephy-auth-data-list", auth_data_list);
         g_object_set_data (G_OBJECT (username_node), "ephy-form-auth", form_auth);
@@ -914,16 +994,125 @@ web_page_document_loaded (WebKitWebPage *web_page,
 }
 
 static void
-web_page_created_callback (WebKitWebExtension *extension,
-                           WebKitWebPage *web_page,
-                           gpointer user_data)
+web_page_uri_changed (WebKitWebPage *web_page,
+                      GParamSpec *param_spec,
+                      EphyWebExtension *extension)
 {
-  g_signal_connect_object (web_page, "send-request",
-                           G_CALLBACK (web_page_send_request),
-                           NULL, 0);
-  g_signal_connect_object (web_page, "document-loaded",
-                           G_CALLBACK (web_page_document_loaded),
-                           NULL, 0);
+  EphyWebOverview *overview = NULL;
+
+  if (g_strcmp0 (webkit_web_page_get_uri (web_page), "ephy-about:overview") == 0)
+    overview = ephy_web_overview_new (web_page, extension->priv->overview_model);
+
+  g_object_set_data_full (G_OBJECT (web_page), "ephy-web-overview", overview, g_object_unref);
+}
+
+static gboolean
+web_page_context_menu (WebKitWebPage *web_page,
+                       WebKitContextMenu *context_menu,
+                       WebKitWebHitTestResult *hit_test_result,
+                       gpointer user_data)
+{
+  char *string;
+  GVariantBuilder builder;
+  WebKitDOMDocument *document = webkit_web_page_get_dom_document (web_page);
+  WebKitDOMDOMWindow *window = webkit_dom_document_get_default_view (document);
+  WebKitDOMDOMSelection *selection = webkit_dom_dom_window_get_selection (window);
+
+  g_object_unref (window);
+
+  if (!selection)
+    return FALSE;
+
+  string = ephy_web_dom_utils_get_selection_as_string (selection);
+  g_object_unref (selection);
+
+  if (!string || *string == '\0')
+  {
+    g_free (string);
+    return FALSE;
+  }
+
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
+  g_variant_builder_add (&builder, "{sv}", "SelectedText", g_variant_new_string (g_strstrip (string)));
+  webkit_context_menu_set_user_data (context_menu,
+                                     g_variant_builder_end (&builder));
+
+  g_free (string);
+
+  return TRUE;
+}
+
+static void
+ephy_web_extension_emit_page_created (EphyWebExtension *extension,
+                                      guint64 page_id)
+{
+  GError *error = NULL;
+
+  g_dbus_connection_emit_signal (extension->priv->dbus_connection,
+                                 NULL,
+                                 EPHY_WEB_EXTENSION_OBJECT_PATH,
+                                 EPHY_WEB_EXTENSION_INTERFACE,
+                                 "PageCreated",
+                                 g_variant_new ("(t)", page_id),
+                                 &error);
+  if (error) {
+    g_warning ("Error emitting signal PageCreated: %s\n", error->message);
+    g_error_free (error);
+  }
+}
+
+static void
+ephy_web_extension_emit_page_created_signals_pending (EphyWebExtension *extension)
+{
+  guint i;
+
+  if (!extension->priv->page_created_signals_pending)
+    return;
+
+  for (i = 0; i < extension->priv->page_created_signals_pending->len; i++) {
+    guint64 page_id;
+
+    page_id = g_array_index (extension->priv->page_created_signals_pending, guint64, i);
+    ephy_web_extension_emit_page_created (extension, page_id);
+  }
+
+  g_array_free (extension->priv->page_created_signals_pending, TRUE);
+  extension->priv->page_created_signals_pending = NULL;
+}
+
+static void
+ephy_web_extension_queue_page_created_signal_emission (EphyWebExtension *extension,
+                                                       guint64 page_id)
+{
+  if (!extension->priv->page_created_signals_pending)
+    extension->priv->page_created_signals_pending = g_array_new (FALSE, FALSE, sizeof (guint64));
+  extension->priv->page_created_signals_pending = g_array_append_val (extension->priv->page_created_signals_pending, page_id);
+}
+
+static void
+ephy_web_extension_page_created_cb (EphyWebExtension *extension,
+                                    WebKitWebPage *web_page)
+{
+  guint64 page_id;
+
+  page_id = webkit_web_page_get_id (web_page);
+  if (extension->priv->dbus_connection)
+    ephy_web_extension_emit_page_created (extension, page_id);
+  else
+    ephy_web_extension_queue_page_created_signal_emission (extension, page_id);
+
+  g_signal_connect (web_page, "send-request",
+                    G_CALLBACK (web_page_send_request),
+                    extension);
+  g_signal_connect (web_page, "document-loaded",
+                    G_CALLBACK (web_page_document_loaded),
+                    extension);
+  g_signal_connect (web_page, "notify::uri",
+                    G_CALLBACK (web_page_uri_changed),
+                    extension);
+  g_signal_connect (web_page, "context-menu",
+                    G_CALLBACK (web_page_context_menu),
+                    extension);
 }
 
 static WebKitWebPage *
@@ -949,7 +1138,7 @@ handle_method_call (GDBusConnection *connection,
                     GDBusMethodInvocation *invocation,
                     gpointer user_data)
 {
-  WebKitWebExtension *web_extension = WEBKIT_WEB_EXTENSION (user_data);
+  EphyWebExtension *extension = EPHY_WEB_EXTENSION (user_data);
 
   if (g_strcmp0 (interface_name, EPHY_WEB_EXTENSION_INTERFACE) != 0)
     return;
@@ -961,7 +1150,7 @@ handle_method_call (GDBusConnection *connection,
     gboolean has_modifed_forms;
 
     g_variant_get (parameters, "(t)", &page_id);
-    web_page = get_webkit_web_page_or_return_dbus_error (invocation, web_extension, page_id);
+    web_page = get_webkit_web_page_or_return_dbus_error (invocation, extension->priv->extension, page_id);
     if (!web_page)
       return;
 
@@ -976,7 +1165,7 @@ handle_method_call (GDBusConnection *connection,
     guint64 page_id;
 
     g_variant_get (parameters, "(t)", &page_id);
-    web_page = get_webkit_web_page_or_return_dbus_error (invocation, web_extension, page_id);
+    web_page = get_webkit_web_page_or_return_dbus_error (invocation, extension->priv->extension, page_id);
     if (!web_page)
       return;
 
@@ -994,7 +1183,7 @@ handle_method_call (GDBusConnection *connection,
     gboolean result;
 
     g_variant_get (parameters, "(ts)", &page_id, &base_uri);
-    web_page = get_webkit_web_page_or_return_dbus_error (invocation, web_extension, page_id);
+    web_page = get_webkit_web_page_or_return_dbus_error (invocation, extension->priv->extension, page_id);
     if (!web_page)
       return;
 
@@ -1013,7 +1202,9 @@ handle_method_call (GDBusConnection *connection,
     EphyEmbedFormAuth *form_auth;
     guint request_id;
     gboolean should_store;
-    GHashTable *requests = get_form_auth_data_save_requests ();
+    GHashTable *requests;
+
+    requests = ephy_web_extension_get_form_auth_data_save_requests (extension);
 
     g_variant_get (parameters, "(ub)", &request_id, &should_store);
 
@@ -1024,8 +1215,63 @@ handle_method_call (GDBusConnection *connection,
     if (should_store)
       store_password (form_auth);
     g_hash_table_remove (requests, GINT_TO_POINTER (request_id));
-  }
+  } else if (g_strcmp0 (method_name, "HistorySetURLs") == 0) {
+    if (extension->priv->overview_model) {
+      GVariantIter iter;
+      GVariant *array;
+      const char *url;
+      const char *title;
+      GList *items = NULL;
 
+      g_variant_get (parameters, "(@a(ss))", &array);
+      g_variant_iter_init (&iter, array);
+
+      while (g_variant_iter_loop (&iter, "(&s&s)", &url, &title))
+        items = g_list_prepend (items, ephy_web_overview_model_item_new (url, title));
+      g_variant_unref (array);
+
+      ephy_web_overview_model_set_urls (extension->priv->overview_model, g_list_reverse (items));
+    }
+    g_dbus_method_invocation_return_value (invocation, NULL);
+  } else if (g_strcmp0 (method_name, "HistorySetURLThumbnail") == 0) {
+    if (extension->priv->overview_model) {
+      const char *url;
+      const char *path;
+
+      g_variant_get (parameters, "(&s&s)", &url, &path);
+      ephy_web_overview_model_set_url_thumbnail (extension->priv->overview_model, url, path);
+    }
+    g_dbus_method_invocation_return_value (invocation, NULL);
+  } else if (g_strcmp0 (method_name, "HistorySetURLTitle") == 0) {
+    if (extension->priv->overview_model) {
+      const char *url;
+      const char *title;
+
+      g_variant_get (parameters, "(&s&s)", &url, &title);
+      ephy_web_overview_model_set_url_title (extension->priv->overview_model, url, title);
+    }
+    g_dbus_method_invocation_return_value (invocation, NULL);
+  } else if (g_strcmp0 (method_name, "HistoryDeleteURL") == 0) {
+    if (extension->priv->overview_model) {
+      const char *url;
+
+      g_variant_get (parameters, "(&s)", &url);
+      ephy_web_overview_model_delete_url (extension->priv->overview_model, url);
+    }
+    g_dbus_method_invocation_return_value (invocation, NULL);
+  } else if (g_strcmp0 (method_name, "HistoryDeleteHost") == 0) {
+    if (extension->priv->overview_model) {
+      const char *host;
+
+      g_variant_get (parameters, "(&s)", &host);
+      ephy_web_overview_model_delete_host (extension->priv->overview_model, host);
+    }
+    g_dbus_method_invocation_return_value (invocation, NULL);
+  } else if (g_strcmp0 (method_name, "HistoryClear") == 0) {
+    if (extension->priv->overview_model)
+      ephy_web_overview_model_clear (extension->priv->overview_model);
+    g_dbus_method_invocation_return_value (invocation, NULL);
+  }
 }
 
 static const GDBusInterfaceVTable interface_vtable = {
@@ -1035,54 +1281,119 @@ static const GDBusInterfaceVTable interface_vtable = {
 };
 
 static void
-bus_acquired_cb (GDBusConnection *connection,
-                 const char *name,
-                 gpointer user_data)
+ephy_web_extension_dispose (GObject *object)
+{
+  EphyWebExtension *extension = EPHY_WEB_EXTENSION (object);
+
+  g_clear_object (&extension->priv->uri_tester);
+  g_clear_object (&extension->priv->overview_model);
+  g_clear_pointer (&extension->priv->form_auth_data_cache,
+                   ephy_form_auth_data_cache_free);
+
+  if (extension->priv->form_auth_data_save_requests) {
+    g_hash_table_destroy (extension->priv->form_auth_data_save_requests);
+    extension->priv->form_auth_data_save_requests = NULL;
+  }
+
+  if (extension->priv->page_created_signals_pending) {
+      g_array_free (extension->priv->page_created_signals_pending, TRUE);
+      extension->priv->page_created_signals_pending = NULL;
+    }
+
+  if (extension->priv->dbus_connection) {
+    g_object_remove_weak_pointer (G_OBJECT (extension->priv->dbus_connection),
+                                  (gpointer *)&extension->priv->dbus_connection);
+    g_dbus_connection_unregister_object (extension->priv->dbus_connection,
+                                         extension->priv->registration_id);
+    extension->priv->registration_id = 0;
+    extension->priv->dbus_connection = NULL;
+  }
+
+  g_clear_object (&extension->priv->extension);
+
+  G_OBJECT_CLASS (ephy_web_extension_parent_class)->dispose (object);
+}
+
+static void
+ephy_web_extension_class_init (EphyWebExtensionClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->dispose = ephy_web_extension_dispose;
+
+  g_type_class_add_private (object_class, sizeof(EphyWebExtensionPrivate));
+}
+
+static void
+ephy_web_extension_init (EphyWebExtension *extension)
+{
+  extension->priv = G_TYPE_INSTANCE_GET_PRIVATE (extension, EPHY_TYPE_WEB_EXTENSION, EphyWebExtensionPrivate);
+  extension->priv->overview_model = ephy_web_overview_model_new ();
+}
+
+static gpointer
+ephy_web_extension_create_instance(gpointer data)
+{
+  return g_object_new (EPHY_TYPE_WEB_EXTENSION, NULL);
+}
+
+EphyWebExtension *
+ephy_web_extension_get (void)
+{
+  static GOnce once_init = G_ONCE_INIT;
+  return EPHY_WEB_EXTENSION (g_once (&once_init, ephy_web_extension_create_instance, NULL));
+}
+
+void
+ephy_web_extension_initialize (EphyWebExtension *extension,
+                               WebKitWebExtension *wk_extension,
+                               const char *dot_dir,
+                               gboolean is_private_profile)
+{
+  g_return_if_fail (EPHY_IS_WEB_EXTENSION (extension));
+
+  if (extension->priv->initialized)
+    return;
+
+  extension->priv->initialized = TRUE;
+
+  extension->priv->extension = g_object_ref (wk_extension);
+  extension->priv->uri_tester = uri_tester_new (dot_dir);
+  if (!is_private_profile)
+    extension->priv->form_auth_data_cache = ephy_form_auth_data_cache_new ();
+
+  g_signal_connect_swapped (extension->priv->extension, "page-created",
+                            G_CALLBACK (ephy_web_extension_page_created_cb),
+                            extension);
+}
+
+void
+ephy_web_extension_dbus_register (EphyWebExtension *extension,
+                                  GDBusConnection *connection)
 {
-  guint registration_id;
   GError *error = NULL;
   static GDBusNodeInfo *introspection_data = NULL;
 
+  g_return_if_fail (EPHY_IS_WEB_EXTENSION (extension));
+  g_return_if_fail (G_IS_DBUS_CONNECTION (connection));
+
   if (!introspection_data)
     introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
 
-  registration_id = g_dbus_connection_register_object (connection,
-                                                       EPHY_WEB_EXTENSION_OBJECT_PATH,
-                                                       introspection_data->interfaces[0],
-                                                       &interface_vtable,
-                                                       g_object_ref (user_data),
-                                                       (GDestroyNotify)g_object_unref,
-                                                       &error);
-  if (!registration_id) {
-    g_warning ("Failed to register object: %s\n", error->message);
+  extension->priv->registration_id =
+    g_dbus_connection_register_object (connection,
+                                       EPHY_WEB_EXTENSION_OBJECT_PATH,
+                                       introspection_data->interfaces[0],
+                                       &interface_vtable,
+                                       extension,
+                                       NULL,
+                                       &error);
+  if (!extension->priv->registration_id) {
+    g_warning ("Failed to register web extension object: %s\n", error->message);
     g_error_free (error);
   } else {
-    dbus_connection = connection;
-    g_object_add_weak_pointer (G_OBJECT (connection), (gpointer *)&dbus_connection);
+    extension->priv->dbus_connection = connection;
+    g_object_add_weak_pointer (G_OBJECT (connection), (gpointer *)&extension->priv->dbus_connection);
+    ephy_web_extension_emit_page_created_signals_pending (extension);
   }
 }
-
-G_MODULE_EXPORT void
-webkit_web_extension_initialize (WebKitWebExtension *extension)
-{
-  char *service_name;
-
-  ephy_debug_init ();
-  uri_tester = uri_tester_new (g_getenv ("EPHY_DOT_DIR"));
-  if (!g_getenv ("EPHY_PRIVATE_PROFILE"))
-    form_auth_data_cache = ephy_form_auth_data_cache_new ();
-
-  g_signal_connect (extension, "page-created",
-                    G_CALLBACK (web_page_created_callback),
-                    NULL);
-
-  service_name = g_strdup_printf ("%s-%s", EPHY_WEB_EXTENSION_SERVICE_NAME, g_getenv ("EPHY_WEB_EXTENSION_ID"));
-  g_bus_own_name (G_BUS_TYPE_SESSION,
-                  service_name,
-                  G_BUS_NAME_OWNER_FLAGS_NONE,
-                  bus_acquired_cb,
-                  NULL, NULL,
-                  g_object_ref (extension),
-                  (GDestroyNotify)g_object_unref);
-  g_free (service_name);
-}
