@@ -30,11 +30,13 @@
 #include "ephy-settings.h"
 #include "ephy-string.h"
 
+#include <errno.h>
 #include <gdk/gdk.h>
 #include <gio/gdesktopappinfo.h>
 #include <gio/gio.h>
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <libxml/xmlreader.h>
 #include <stdlib.h>
@@ -228,7 +230,7 @@ ephy_file (const char *filename)
 
 	static const char * const paths[] =
 	{
-#ifdef GNOME_ENABLE_DEBUG
+#ifndef NDEBUG
 		TOP_SRC_DATADIR "/",
 		TOP_SRC_DATADIR "/icons/",
 		TOP_SRC_DATADIR "/pages/",
@@ -429,15 +431,10 @@ ephy_file_helpers_shutdown (void)
 	{
 		if (!keep_directory)
 		{
-			GFile *tmp_dir_file;
-			tmp_dir_file = g_file_new_for_path (tmp_dir);
-
 			/* recursively delete the contents and the
 			 * directory */
 			LOG ("shutdown: delete tmp_dir %s", tmp_dir);
-			ephy_file_delete_dir_recursively (tmp_dir_file,
-							  NULL);
-			g_object_unref (tmp_dir_file);
+			ephy_file_delete_dir_recursively (tmp_dir, NULL);
 		}
 
 		g_free (tmp_dir);
@@ -817,6 +814,38 @@ ephy_file_launch_handler (const char *mime_type,
 	return ret;
 }
 
+gboolean
+ephy_file_open_uri_in_default_browser (const char *uri,
+				       guint32 timestamp,
+				       GdkScreen *screen)
+{
+	GdkAppLaunchContext *context;
+	GAppInfo *appinfo;
+	GList uris;
+	gboolean retval = TRUE;
+	GError *error = NULL;
+
+	context = gdk_display_get_app_launch_context (screen ? gdk_screen_get_display (screen) : gdk_display_get_default ());
+	gdk_app_launch_context_set_screen (context, screen);
+	gdk_app_launch_context_set_timestamp (context, timestamp);
+
+	appinfo = g_app_info_get_default_for_type ("x-scheme-handler/http", TRUE);
+	uris.data = (gpointer)uri;
+	uris.next = uris.prev = NULL;
+
+	if (!g_app_info_launch_uris (appinfo, &uris, G_APP_LAUNCH_CONTEXT (context), &error))
+	{
+		g_warning ("Failed to launch %s: %s", uri, error->message);
+		g_error_free (error);
+		retval = FALSE;
+	}
+
+	g_object_unref (context);
+	g_object_unref (appinfo);
+
+	return retval;
+}
+
 /**
  * ephy_file_browse_to:
  * @file: a #GFile
@@ -845,56 +874,62 @@ ephy_file_browse_to (GFile *file,
  * Returns: %TRUE if delete succeeded
  **/
 gboolean
-ephy_file_delete_dir_recursively (GFile *directory, GError **error)
+ephy_file_delete_dir_recursively (const char *directory, GError **error)
 {
-	GFileEnumerator *children = NULL;
-	GFileInfo *info;
-	gboolean ret = FALSE;
+	GDir* dir;
+	const char* file_name;
+	gboolean failed = FALSE;
 
-	children = g_file_enumerate_children (directory,
-					      "standard::name,standard::type",
-					      0, NULL, error);
-	if (children == NULL || error)
-		goto out;
+	dir = g_dir_open (directory, 0, error);
+	if (!dir)
+		return FALSE;
 
-	info = g_file_enumerator_next_file (children, NULL, error);
-	while (info || error) {
-		GFile *child;
-		const char *name;
-		GFileType type;
+	file_name = g_dir_read_name (dir);
+	while (file_name && !failed) {
+		char *file_path;
 
-		if (error)
-			goto out;
+		file_path = g_build_filename (directory, file_name, NULL);
+		if (g_file_test (file_path, G_FILE_TEST_IS_DIR))
+		{
+			failed = !ephy_file_delete_dir_recursively (file_path, error);
+		}
+		else
+		{
+			int result = g_unlink (file_path);
 
-		name = g_file_info_get_name (info);
-		child = g_file_get_child (directory, name);
-		type = g_file_info_get_file_type (info);
+			if (result == -1)
+			{
+				int errsv = errno;
 
-		LOG ("ephy-file-delete-dir: delete child %s", name);
-		if (type == G_FILE_TYPE_DIRECTORY)
-			ret = ephy_file_delete_dir_recursively (child, error);
-		else if (type == G_FILE_TYPE_REGULAR)
-			ret = g_file_delete (child, NULL, error);
+				g_set_error (error, G_IO_ERROR,
+					     g_io_error_from_errno (errsv),
+					     "Error removing file %s: %s",
+					     file_path, g_strerror (errsv));
+				failed = TRUE;
+			}
+		}
+		g_free (file_path);
+		file_name = g_dir_read_name (dir);
+	}
+	g_dir_close (dir);
 
-		g_object_unref (info);
-		g_object_unref (child);
+	if (!failed)
+	{
+		int result = g_rmdir (directory);
 
-		if (!ret)
-			goto out;
+		if (result == -1)
+		{
+			int errsv = errno;
 
-		info = g_file_enumerator_next_file (children, NULL, error);
+			g_set_error (error, G_IO_ERROR,
+				     g_io_error_from_errno (errsv),
+				     "Error removing directory %s: %s",
+				     directory, g_strerror (errsv));
+			failed = TRUE;
+		}
 	}
 
-	ret = TRUE;
-
-	LOG ("ephy-file-delete-dir: delete successful");
-	g_file_delete (directory, NULL, error);
-
-out:
-	if (children)
-		g_object_unref (children);
-
-	return ret;
+	return !failed;
 }
 
 /**
@@ -926,6 +961,43 @@ ephy_file_delete_uri (const char *uri)
 	g_object_unref (file);
 }
 
+/**
+ * ephy_file_move_uri
+ * @source_uri: URI of the file to be moved
+ * @dest_uri: URI of the destination
+ *
+ * Move from source_uri to dest_uri, overwriting if necessary.
+ *
+ * Returns: %TRUE on successful move, %FALSE otherwise.
+ */
+gboolean
+ephy_file_move_uri (const char *source_uri, const char *dest_uri)
+{
+	GFile *src;
+	GFile *dest;
+	gboolean ret;
+
+	g_return_val_if_fail (source_uri && dest_uri, FALSE);
+
+	src = g_file_new_for_uri (source_uri);
+	dest = g_file_new_for_uri (dest_uri);
+
+	ret = g_file_move (src, dest, G_FILE_COPY_OVERWRITE | G_FILE_COPY_ALL_METADATA,
+			   NULL, NULL, NULL, NULL);
+
+	if (ret == TRUE)
+	{
+		LOG ("Moved file '%s' to '%s'", source_uri, dest_uri);
+	}
+	else
+	{
+		LOG ("Couldn't move file '%s' to '%s'", source_uri, dest_uri);
+	}
+	g_object_unref (src);
+	g_object_unref (dest);
+	return ret;
+}
+
 /**
  * ephy_file_create_data_uri_for_filename:
  * @filename: the filename of a local path
@@ -998,3 +1070,27 @@ ephy_sanitize_filename (char *filename)
 
 	return g_strdelimit (filename, G_DIR_SEPARATOR_S, '_');
 }
+
+void
+ephy_open_incognito_window (const char *uri)
+{
+	char *command;
+	GError *error = NULL;
+
+	command = g_strdup_printf ("epiphany --incognito-mode --profile %s ", ephy_dot_dir ());
+
+	if (uri) {
+		char *str = g_strconcat (command, uri, NULL);
+		g_free (command);
+		command = str;
+	}
+
+	g_spawn_command_line_async (command, &error);
+
+	if (error) {
+		g_warning ("Couldn't open link in incognito window: %s", error->message);
+		g_error_free (error);
+	}
+
+	g_free (command);
+}
