@@ -248,4 +248,38 @@ bail:
   soup_uri_free (uri);
   return ret;
 }
+
+char *
+ephy_uri_safe_unescape (const char *uri_string)
+{
+  char *decoded_uri;
+
+  /* This function is not null-safe since it is mostly used in scenarios where
+   * passing or returning null would typically lead to a security issue. */
+  g_return_val_if_fail (uri_string, g_strdup (""));
+
+  /* Protect against escaped null characters and escaped slashes. */
+  decoded_uri = g_uri_unescape_string (uri_string, "/");
+  return decoded_uri ? decoded_uri : g_strdup (uri_string);
+}
+
+char *
+ephy_uri_normalize (const char *uri_string)
+{
+  SoupURI *uri;
+  char *encoded_uri;
+
+  if (!uri_string || !*uri_string)
+    return NULL;
+
+  uri = soup_uri_new (uri_string);
+  if (!uri)
+    return g_strdup (uri_string);
+
+  encoded_uri = soup_uri_normalize (uri_string, NULL);
+  soup_uri_free (uri);
+
+  return encoded_uri;
+}
+
 /* vim: set sw=2 ts=2 sts=2 et: */
