@@ -45,7 +45,7 @@ ephy_history_service_initialize_urls_table (EphyHistoryService *self)
     "hidden_from_overview INTEGER DEFAULT 0)", &error);
 
   if (error) {
-    g_error("Could not create urls table: %s", error->message);
+    g_warning ("Could not create urls table: %s", error->message);
     g_error_free (error);
     return FALSE;
   }
@@ -79,7 +79,7 @@ ephy_history_service_get_url_row (EphyHistoryService *self, const char *url_stri
   }
 
   if (error) {
-    g_error ("Could not build urls table query statement: %s", error->message);
+    g_warning ("Could not build urls table query statement: %s", error->message);
     g_error_free (error);
     return NULL;
   }
@@ -90,7 +90,7 @@ ephy_history_service_get_url_row (EphyHistoryService *self, const char *url_stri
     ephy_sqlite_statement_bind_string (statement, 0, url_string, &error);
   }
   if (error) {
-    g_error ("Could not build urls table query statement: %s", error->message);
+    g_warning ("Could not build urls table query statement: %s", error->message);
     g_error_free (error);
     g_object_unref (statement);
     return NULL;
@@ -138,7 +138,7 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
     "INSERT INTO urls (url, title, visit_count, typed_count, last_visit_time, host) "
     " VALUES (?, ?, ?, ?, ?, ?)", &error);
   if (error) {
-    g_error ("Could not build urls table addition statement: %s", error->message);
+    g_warning ("Could not build urls table addition statement: %s", error->message);
     g_error_free (error);
     return;
   }
@@ -149,7 +149,7 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
       ephy_sqlite_statement_bind_int (statement, 3, url->typed_count, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 4, url->last_visit_time, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 5, url->host->id, &error) == FALSE) {
-    g_error ("Could not insert URL into urls table: %s", error->message);
+    g_warning ("Could not insert URL into urls table: %s", error->message);
     g_error_free (error);
     g_object_unref (statement);
     return;
@@ -157,7 +157,7 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
 
   ephy_sqlite_statement_step (statement, &error);
   if (error) {
-    g_error ("Could not insert URL into urls table: %s", error->message);
+    g_warning ("Could not insert URL into urls table: %s", error->message);
     g_error_free (error);
   } else {
     url->id = ephy_sqlite_connection_get_last_insert_id (priv->history_database);
@@ -180,7 +180,7 @@ ephy_history_service_update_url_row (EphyHistoryService *self, EphyHistoryURL *u
     "UPDATE urls SET title=?, visit_count=?, typed_count=?, last_visit_time=?, hidden_from_overview=?, thumbnail_update_time=? "
     "WHERE id=?", &error);
   if (error) {
-    g_error ("Could not build urls table modification statement: %s", error->message);
+    g_warning ("Could not build urls table modification statement: %s", error->message);
     g_error_free (error);
     return;
   }
@@ -192,7 +192,7 @@ ephy_history_service_update_url_row (EphyHistoryService *self, EphyHistoryURL *u
       ephy_sqlite_statement_bind_int (statement, 4, url->hidden, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 5, url->thumbnail_time, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 6, url->id, &error) == FALSE) {
-    g_error ("Could not modify URL in urls table: %s", error->message);
+    g_warning ("Could not modify URL in urls table: %s", error->message);
     g_error_free (error);
     g_object_unref (statement);
     return;
@@ -200,7 +200,7 @@ ephy_history_service_update_url_row (EphyHistoryService *self, EphyHistoryURL *u
 
   ephy_sqlite_statement_step (statement, &error);
   if (error) {
-    g_error ("Could not modify URL in urls table: %s", error->message);
+    g_warning ("Could not modify URL in urls table: %s", error->message);
     g_error_free (error);
   }
   g_object_unref (statement);
@@ -267,6 +267,9 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   if (query->ignore_hidden)
     statement_str = g_string_append (statement_str, "urls.hidden_from_overview = 0 AND ");
 
+  if (query->ignore_local)
+    statement_str = g_string_append (statement_str, "urls.url LIKE 'http%' AND ");
+
   if (query->host > 0)
     statement_str = g_string_append (statement_str, "urls.host = ? AND ");
 
@@ -313,14 +316,14 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   g_string_free (statement_str, TRUE);
 
   if (error) {
-    g_error ("Could not build urls table query statement: %s", error->message);
+    g_warning ("Could not build urls table query statement: %s", error->message);
     g_error_free (error);
     return NULL;
   }
 
   if (query->from > 0) {
     if (ephy_sqlite_statement_bind_int (statement, i++, (int)query->from, &error) == FALSE) {
-      g_error ("Could not build urls table query statement: %s", error->message);
+      g_warning ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
       g_object_unref (statement);
       return NULL;
@@ -328,7 +331,7 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   }
   if (query->to > 0) {
     if (ephy_sqlite_statement_bind_int (statement, i++, (int)query->to, &error) == FALSE) {
-      g_error ("Could not build urls table query statement: %s", error->message);
+      g_warning ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
       g_object_unref (statement);
       return NULL;
@@ -336,7 +339,7 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   }
   if (query->host > 0) {
     if (ephy_sqlite_statement_bind_int (statement, i++, (int)query->host, &error) == FALSE) {
-      g_error ("Could not build urls table query statement: %s", error->message);
+      g_warning ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
       g_object_unref (statement);
       return NULL;
@@ -345,14 +348,14 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   for (substring = query->substring_list; substring != NULL; substring = substring->next) {
     char *string = ephy_sqlite_create_match_pattern (substring->data);
     if (ephy_sqlite_statement_bind_string (statement, i++, string, &error) == FALSE) {
-      g_error ("Could not build urls table query statement: %s", error->message);
+      g_warning ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
       g_object_unref (statement);
       g_free (string);
       return NULL;
     }
     if (ephy_sqlite_statement_bind_string (statement, i++, string + 2, &error) == FALSE) {
-      g_error ("Could not build urls table query statement: %s", error->message);
+      g_warning ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
       g_object_unref (statement);
       g_free (string);
@@ -363,7 +366,7 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
 
   if (query->limit)
     if (ephy_sqlite_statement_bind_int (statement, i++, query->limit, &error) == FALSE) {
-      g_error ("Could not build urls table query statement: %s", error->message);
+      g_warning ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
       g_object_unref (statement);
       return NULL;
@@ -375,7 +378,7 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   urls = g_list_reverse (urls);
 
   if (error) {
-    g_error ("Could not execute urls table query statement: %s", error->message);
+    g_warning ("Could not execute urls table query statement: %s", error->message);
     g_error_free (error);
     g_object_unref (statement);
     g_list_free_full (urls, (GDestroyNotify)ephy_history_url_free);
@@ -409,7 +412,7 @@ ephy_history_service_delete_url (EphyHistoryService *self, EphyHistoryURL *url)
   g_free (sql_statement);
 
   if (error) {
-    g_error ("Could not build urls table query statement: %s", error->message);
+    g_warning ("Could not build urls table query statement: %s", error->message);
     g_error_free (error);
     return;
   }
@@ -420,7 +423,7 @@ ephy_history_service_delete_url (EphyHistoryService *self, EphyHistoryURL *url)
     ephy_sqlite_statement_bind_string (statement, 0, url->url, &error);
 
   if (error) {
-    g_error ("Could not build urls table query statement: %s", error->message);
+    g_warning ("Could not build urls table query statement: %s", error->message);
     g_error_free (error);
     g_object_unref (statement);
     return;
@@ -428,7 +431,7 @@ ephy_history_service_delete_url (EphyHistoryService *self, EphyHistoryURL *url)
 
   ephy_sqlite_statement_step (statement, &error);
   if (error) {
-    g_error ("Could not modify URL in urls table: %s", error->message);
+    g_warning ("Could not modify URL in urls table: %s", error->message);
     g_error_free (error);
   }
   g_object_unref (statement);
