@@ -31,13 +31,17 @@
 #include "ephy-gui.h"
 #include "ephy-lib-type-builtins.h"
 #include "ephy-signal-accumulator.h"
+#include "ephy-uri-helpers.h"
 
-#include <libsoup/soup.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <string.h>
+#if 0
+/* FIXME: Refactor the DNS prefetch, this is a layering violation */
+#include <libsoup/soup.h>
 #include <webkit2/webkit2.h>
+#endif
 
 /**
  * SECTION:ephy-location-entry
@@ -53,7 +57,6 @@ struct _EphyLocationEntryPrivate
 {
 	GdkPixbuf *favicon;
 	GtkTreeModel *model;
-	GtkEntryCompletion *completion;
 
 	GSList *search_terms;
 
@@ -207,16 +210,57 @@ ephy_location_entry_get_preferred_width (GtkWidget       *widget,
 		*natural_width = 848;
 }
 
+static void
+ephy_location_entry_copy_clipboard (GtkEntry *entry)
+{
+	char *text;
+	gint start;
+	gint end;
+
+	if (!gtk_editable_get_selection_bounds (GTK_EDITABLE (entry), &start, &end))
+		return;
+
+	text = gtk_editable_get_chars (GTK_EDITABLE (entry), start, end);
+
+	if (start == 0)
+	{
+		char *tmp = text;
+		text = ephy_uri_normalize (tmp);
+		g_free (tmp);
+	}
+
+	gtk_clipboard_set_text (gtk_widget_get_clipboard (GTK_WIDGET (entry),
+							  GDK_SELECTION_CLIPBOARD),
+							  text, -1);
+	g_free (text);
+}
+
+static void
+ephy_location_entry_cut_clipboard (GtkEntry *entry)
+{
+	if (!gtk_editable_get_editable (GTK_EDITABLE (entry)))
+	{
+		gtk_widget_error_bell (GTK_WIDGET (entry));
+		return;
+	}
+
+	ephy_location_entry_copy_clipboard (entry);
+	gtk_editable_delete_selection (GTK_EDITABLE (entry));
+}
+
 static void
 ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+	GtkEntryClass *entry_class = GTK_ENTRY_CLASS (klass);
 
 	object_class->get_property = ephy_location_entry_get_property;
 	object_class->set_property = ephy_location_entry_set_property;
 	object_class->finalize = ephy_location_entry_finalize;
 	widget_class->get_preferred_width = ephy_location_entry_get_preferred_width;
+	entry_class->copy_clipboard = ephy_location_entry_copy_clipboard;
+	entry_class->cut_clipboard = ephy_location_entry_cut_clipboard;
 
 	/**
 	* EphyLocationEntry:location:
@@ -957,6 +1001,8 @@ ephy_location_entry_new (void)
 	return GTK_WIDGET (g_object_new (EPHY_TYPE_LOCATION_ENTRY, NULL));
 }
 
+#if 0
+/* FIXME: Refactor the DNS prefetch, this is a layering violation */
 typedef struct {
 	SoupURI *uri;
 	EphyLocationEntry *entry;
@@ -973,10 +1019,10 @@ free_prefetch_helper (PrefetchHelper *helper)
 static gboolean
 do_dns_prefetch (PrefetchHelper *helper)
 {
-	WebKitWebContext *context = webkit_web_context_get_default ();
+	EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
 	if (helper->uri)
-		webkit_web_context_prefetch_dns (context, helper->uri->host);
+		webkit_web_context_prefetch_dns (ephy_embed_shell_get_web_context (shell), helper->uri->host);
 
 	helper->entry->priv->dns_prefetch_handler = 0;
 
@@ -1008,6 +1054,7 @@ schedule_dns_prefetch (EphyLocationEntry *entry, guint interval, const gchar *ur
 				    (GDestroyNotify) free_prefetch_helper);
 	g_source_set_name_by_id (entry->priv->dns_prefetch_handler, "[epiphany] do_dns_prefetch");
 }
+#endif
 
 static gboolean
 cursor_on_match_cb  (GtkEntryCompletion *completion,
@@ -1031,76 +1078,16 @@ cursor_on_match_cb  (GtkEntryCompletion *completion,
 	gtk_editable_set_position (GTK_EDITABLE (entry), -1);
 	le->priv->block_update = FALSE;
 
+#if 0
+/* FIXME: Refactor the DNS prefetch, this is a layering violation */
 	schedule_dns_prefetch (le, 250, (const gchar*) url);
+#endif
 
 	g_free (url);
 
 	return TRUE;
 }
 
-static char *
-rgba_to_hex (GdkRGBA *color)
-{
-	char *hex;
-
-	hex = g_strdup_printf ("#%04X%04X%04X",
-			       (guint)(color->red * (gdouble)65535),
-			       (guint)(color->green * (gdouble)65535),
-			       (guint)(color->blue * (gdouble)65535));
-	return hex;
-}
-
-static void
-textcell_data_func (GtkCellLayout *cell_layout,
-		    GtkCellRenderer *cell,
-		    GtkTreeModel *tree_model,
-		    GtkTreeIter *iter,
-		    gpointer data)
-{
-	GtkWidget *entry;
-	EphyLocationEntryPrivate *priv;
-	char *ctext;
-	char *title;
-	char *url;
-	GtkStyleContext *style;
-	GValue text = { 0, };
-
-	entry = GTK_WIDGET (data);
-	priv = EPHY_LOCATION_ENTRY (data)->priv;
-	gtk_tree_model_get (tree_model, iter,
-			priv->text_col, &title,
-			priv->url_col, &url,
-			-1);
-
-	if (url)
-	{
-		GdkRGBA color;
-		char *color_text;
-
-		sanitize_location (&url);
-
-		style = gtk_widget_get_style_context (entry);
-		gtk_style_context_get_color (style, GTK_STATE_FLAG_INSENSITIVE,
-					     &color);
-
-		color_text = rgba_to_hex (&color);
-		ctext = g_markup_printf_escaped ("%s\n<span font-size=\"small\" color=\"%s\">%s</span>", title, color_text, url);
-		g_free (color_text);
-	        g_free (title);
-	}
-	else
-	{
-		ctext = title;
-	}
-
-	g_value_init (&text, G_TYPE_STRING);
-	g_value_take_string (&text, ctext);
-	g_object_set_property (G_OBJECT (cell), "markup", &text);
-	g_value_unset (&text);
-
-	g_free (url);
-}
-
 static void
 extracell_data_func (GtkCellLayout *cell_layout,
 			GtkCellRenderer *cell,
@@ -1235,7 +1222,7 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 
 #define ICON_PADDING_LEFT 5
 #define ICON_CONTENT_WIDTH 16
-#define ICON_PADDING_RIGHT 4
+#define ICON_PADDING_RIGHT 9
 
 #define ICON_CONTENT_HEIGHT 16
 
@@ -1256,7 +1243,7 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
 				    cell, TRUE);
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
-				       cell, "text", text_col);
+				       cell, "markup", text_col);
 
 	/* Pixel-perfect aligment with the text in the location entry.
 	 * See above.
@@ -1276,11 +1263,6 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 	gtk_cell_renderer_set_fixed_size (cell, 1, -1);
 	gtk_cell_renderer_text_set_fixed_height_from_font (GTK_CELL_RENDERER_TEXT (cell), 2);
 
-	gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (completion),
-					    cell, textcell_data_func,
-					    entry,
-					    NULL);
-
 	cell = gtk_cell_renderer_pixbuf_new ();
 	g_object_set (cell, "follow-state", TRUE, NULL);
 	gtk_cell_layout_pack_end (GTK_CELL_LAYOUT (completion),
@@ -1300,8 +1282,6 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 			  G_CALLBACK (cursor_on_match_cb), entry);
 
 	gtk_entry_set_completion (GTK_ENTRY (entry), completion);
-
-	priv->completion = completion;
 	g_object_unref (completion);
 }
 
