@@ -53,11 +53,9 @@ struct _EphyNotebookPrivate
 	GList *focused_pages;
 	guint tabs_vis_notifier_id;
 
-	guint show_tabs : 1;
+	guint tabs_allowed : 1;
 };
 
-static void ephy_notebook_init		 (EphyNotebook *notebook);
-static void ephy_notebook_class_init	 (EphyNotebookClass *klass);
 static void ephy_notebook_finalize	 (GObject *object);
 static int  ephy_notebook_insert_page	 (GtkNotebook *notebook,
 					  GtkWidget *child,
@@ -77,7 +75,7 @@ static const GtkTargetEntry url_drag_types [] =
 enum
 {
 	PROP_0,
-	PROP_SHOW_TABS
+	PROP_TABS_ALLOWED
 };
 
 enum
@@ -103,8 +101,8 @@ ephy_notebook_get_property (GObject *object,
 
 	switch (prop_id)
 	{
-		case PROP_SHOW_TABS:
-			g_value_set_boolean (value, priv->show_tabs);
+		case PROP_TABS_ALLOWED:
+			g_value_set_boolean (value, priv->tabs_allowed);
 			break;
 	}
 }
@@ -119,8 +117,8 @@ ephy_notebook_set_property (GObject *object,
 
 	switch (prop_id)
 	{
-		case PROP_SHOW_TABS:
-			ephy_notebook_set_show_tabs (notebook, g_value_get_boolean (value));
+		case PROP_TABS_ALLOWED:
+			ephy_notebook_set_tabs_allowed (notebook, g_value_get_boolean (value));
 			break;
 	}
 }
@@ -152,8 +150,8 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 			      GTK_TYPE_WIDGET /* Can't use an interface type here */);
 
 	g_object_class_install_property (object_class,
-					 PROP_SHOW_TABS,
-					 g_param_spec_boolean ("show-tabs", NULL, NULL,
+					 PROP_TABS_ALLOWED,
+					 g_param_spec_boolean ("tabs-allowed", NULL, NULL,
 							       TRUE,
 							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
@@ -209,7 +207,6 @@ is_in_notebook_window (EphyNotebook *notebook,
 static gint
 find_tab_num_at_pos (EphyNotebook *notebook, gint abs_x, gint abs_y)
 {
-	GtkPositionType tab_pos;
 	int page_num = 0;
 	GtkNotebook *nb = GTK_NOTEBOOK (notebook);
 	GtkWidget *page;
@@ -221,8 +218,6 @@ find_tab_num_at_pos (EphyNotebook *notebook, gint abs_x, gint abs_y)
 		return NOT_IN_APP_WINDOWS;
 	}
 
-	tab_pos = gtk_notebook_get_tab_pos (nb);
-
 	while ((page = gtk_notebook_get_nth_page (nb, page_num)))
 	{
 		GtkWidget *tab;
@@ -246,15 +241,7 @@ find_tab_num_at_pos (EphyNotebook *notebook, gint abs_x, gint abs_y)
 		max_x = x_root + allocation.x + allocation.width;
 		max_y = y_root + allocation.y + allocation.height;
 
-		if (((tab_pos == GTK_POS_TOP)
-		     || (tab_pos == GTK_POS_BOTTOM))
-		    &&(abs_x<=max_x))
-		{
-			return page_num;
-		}
-		else if (((tab_pos == GTK_POS_LEFT)
-			  || (tab_pos == GTK_POS_RIGHT))
-			 && (abs_y<=max_y))
+		if (abs_y <= max_y && abs_x <= max_x)
 		{
 			return page_num;
 		}
@@ -381,8 +368,12 @@ notebook_drag_data_received_cb (GtkWidget* widget,
 	       
 		text = (char *) gtk_selection_data_get_text (selection_data);
 		if (text != NULL) {
-			ephy_link_open (EPHY_LINK (notebook), text, embed,
+			char *address;
+
+			address = ephy_embed_utils_normalize_or_autosearch_address (text);
+			ephy_link_open (EPHY_LINK (notebook), address, embed,
 					embed ? 0 : EPHY_LINK_NEW_TAB);
+			g_free (address);
 			g_free (text);
 		}
 	}
@@ -396,23 +387,26 @@ static void
 update_tabs_visibility (EphyNotebook *nb,
 			gboolean before_inserting)
 {
-	EphyNotebookPrivate *priv = nb->priv;
 	EphyEmbedShellMode mode;
-	gboolean show_tabs;
+	gboolean show_tabs = FALSE;
 	guint num;
+	EphyPrefsUITabsBarVisibilityPolicy policy;
 
 	mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (ephy_shell_get_default ()));
 	num = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb));
 
 	if (before_inserting) num++;
 
-	show_tabs = mode != EPHY_EMBED_SHELL_MODE_APPLICATION &&
-		(g_settings_get_boolean (EPHY_SETTINGS_UI,
-					 EPHY_PREFS_UI_ALWAYS_SHOW_TABS_BAR)
-		 || num > 1) &&
-		priv->show_tabs == TRUE;
+	policy = g_settings_get_enum (EPHY_SETTINGS_UI,
+				      EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY);
 
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (nb), show_tabs);
+	if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION &&
+	    ((policy == EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY_MORE_THAN_ONE && num > 1) ||
+	     policy == EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY_ALWAYS))
+		show_tabs = TRUE;
+
+	/* Only show the tabs when the "tabs-allowed" property is TRUE. */
+	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (nb), nb->priv->tabs_allowed && show_tabs);
 }
 
 static void
@@ -437,7 +431,7 @@ ephy_notebook_init (EphyNotebook *notebook)
 	gtk_notebook_set_show_tabs (gnotebook, FALSE);
 	gtk_notebook_set_group_name (gnotebook, EPHY_NOTEBOOK_TAB_GROUP_ID);
 
-	priv->show_tabs = TRUE;
+	priv->tabs_allowed = TRUE;
 
 	g_signal_connect (notebook, "button-press-event",
 			  (GCallback)button_press_cb, NULL);
@@ -455,7 +449,7 @@ ephy_notebook_init (EphyNotebook *notebook)
 	gtk_drag_dest_add_text_targets (widget);
 
 	g_signal_connect (EPHY_SETTINGS_UI,
-			  "changed::" EPHY_PREFS_UI_ALWAYS_SHOW_TABS_BAR,
+			  "changed::" EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY,
 			  G_CALLBACK (show_tabs_changed_cb), notebook);
 }
 
@@ -465,6 +459,9 @@ ephy_notebook_finalize (GObject *object)
 	EphyNotebook *notebook = EPHY_NOTEBOOK (object);
 	EphyNotebookPrivate *priv = notebook->priv;
 
+	g_signal_handlers_disconnect_by_func (EPHY_SETTINGS_UI,
+					      show_tabs_changed_cb,
+					      notebook);
 	g_list_free (priv->focused_pages);
 
 	G_OBJECT_CLASS (ephy_notebook_parent_class)->finalize (object);
@@ -474,12 +471,14 @@ static void
 sync_load_status (EphyWebView *view, GParamSpec *pspec, GtkWidget *proxy)
 {
 	GtkWidget *spinner, *icon;
+	EphyEmbed *embed;
 
 	spinner = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "spinner"));
 	icon = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "icon"));
 	g_return_if_fail (spinner != NULL && icon != NULL);
 
-	if (ephy_web_view_is_loading (view))
+	embed = EPHY_GET_EMBED_FROM_EPHY_WEB_VIEW (view);
+	if (ephy_web_view_is_loading (view) && !ephy_embed_has_load_pending (embed))
 	{
 		gtk_widget_hide (icon);
 		gtk_widget_show (spinner);
@@ -493,6 +492,12 @@ sync_load_status (EphyWebView *view, GParamSpec *pspec, GtkWidget *proxy)
 	}
 }
 
+static void
+load_changed_cb (EphyWebView *view, WebKitLoadEvent load_event, GtkWidget *proxy)
+{
+	sync_load_status (view, NULL, proxy);
+}
+
 static void
 sync_icon (EphyWebView *view,
 	   GParamSpec *pspec,
@@ -502,13 +507,21 @@ sync_icon (EphyWebView *view,
 }
 
 static void
-sync_label (EphyWebView *view, GParamSpec *pspec, GtkWidget *label)
+sync_label (EphyEmbed *embed, GParamSpec *pspec, GtkWidget *label)
 {
 	const char *title;
 
-	title = ephy_web_view_get_title (view);
-
+	title = ephy_embed_get_title (embed);
 	gtk_label_set_text (GTK_LABEL (label), title);
+	gtk_widget_set_tooltip_text (label, title);
+}
+
+static void
+sync_is_playing_audio (WebKitWebView *view,
+		       GParamSpec *pspec,
+		       GtkWidget *speaker_icon)
+{
+	gtk_widget_set_visible (speaker_icon, webkit_web_view_is_playing_audio (view));
 }
 
 static void
@@ -528,14 +541,18 @@ tab_label_style_set_cb (GtkWidget *hbox,
 	PangoFontMetrics *metrics;
 	PangoContext *context;
 	GtkStyleContext *style;
+	PangoFontDescription *font_desc;
 	GtkWidget *button;
 	int char_width, h, w;
 
 	context = gtk_widget_get_pango_context (hbox);
 	style = gtk_widget_get_style_context (hbox);
+	gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL,
+			       "font", &font_desc, NULL);
 	metrics = pango_context_get_metrics (context,
-					     gtk_style_context_get_font (style, GTK_STATE_FLAG_NORMAL),
+					     font_desc,
 					     pango_context_get_language (context));
+	pango_font_description_free (font_desc);
 	char_width = pango_font_metrics_get_approximate_digit_width (metrics);
 	pango_font_metrics_unref (metrics);
 
@@ -552,13 +569,19 @@ tab_label_style_set_cb (GtkWidget *hbox,
 static GtkWidget *
 build_tab_label (EphyNotebook *nb, EphyEmbed *embed)
 {
-	GtkWidget *hbox, *label, *close_button, *image, *spinner, *icon;
+	GtkWidget *hbox, *label, *close_button, *image, *spinner, *icon, *speaker_icon;
+	GtkWidget *box;
 	EphyWebView *view;
 
+	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
+	gtk_widget_show (box);
+
 	/* set hbox spacing and label padding (see below) so that there's an
 	 * equal amount of space around the label */
-	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 	gtk_widget_show (hbox);
+	gtk_widget_set_halign (hbox, GTK_ALIGN_CENTER);
+	gtk_box_pack_start (GTK_BOX (box), hbox, TRUE, TRUE, 0);
 
 	/* setup load feedback */
 	spinner = gtk_spinner_new ();
@@ -573,11 +596,15 @@ build_tab_label (EphyNotebook *nb, EphyEmbed *embed)
 	label = gtk_label_new (NULL);
 	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
 	gtk_label_set_single_line_mode (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_misc_set_padding (GTK_MISC (label), 0, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	gtk_widget_show (label);
 
+	/* setup speaker icon */
+	speaker_icon = gtk_image_new_from_icon_name ("audio-volume-high-symbolic",
+						     GTK_ICON_SIZE_MENU);
+	gtk_box_pack_start (GTK_BOX (hbox), speaker_icon, FALSE, FALSE, 0);
+
 	/* setup close button */
 	close_button = gtk_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (close_button),
@@ -596,51 +623,56 @@ build_tab_label (EphyNotebook *nb, EphyEmbed *embed)
 	gtk_container_add (GTK_CONTAINER (close_button), image);
 	gtk_widget_show (image);
 
-	gtk_box_pack_start (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (box), close_button, FALSE, FALSE, 0);
 	gtk_widget_show (close_button);
 
 	/* Set minimal size */
-	g_signal_connect (hbox, "style-set",
+	g_signal_connect (box, "style-set",
 			  G_CALLBACK (tab_label_style_set_cb), NULL);
 
 	/* Set up drag-and-drop target */
-	g_signal_connect (hbox, "drag-data-received",
+	g_signal_connect (box, "drag-data-received",
 			  G_CALLBACK (notebook_drag_data_received_cb), embed);
-	gtk_drag_dest_set (hbox, GTK_DEST_DEFAULT_ALL,
+	gtk_drag_dest_set (box, GTK_DEST_DEFAULT_ALL,
 			   url_drag_types, G_N_ELEMENTS (url_drag_types),
 			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
-	gtk_drag_dest_add_text_targets (hbox);
+	gtk_drag_dest_add_text_targets (box);
 
-	g_object_set_data (G_OBJECT (hbox), "label", label);
-	g_object_set_data (G_OBJECT (hbox), "spinner", spinner);
-	g_object_set_data (G_OBJECT (hbox), "icon", icon);
-	g_object_set_data (G_OBJECT (hbox), "close-button", close_button);
+	g_object_set_data (G_OBJECT (box), "label", label);
+	g_object_set_data (G_OBJECT (box), "spinner", spinner);
+	g_object_set_data (G_OBJECT (box), "icon", icon);
+	g_object_set_data (G_OBJECT (box), "close-button", close_button);
+	g_object_set_data (G_OBJECT (box), "speaker-icon", speaker_icon);
 
 	/* Hook the label up to the tab properties */
 	view = ephy_embed_get_web_view (embed);
 	sync_icon (view, NULL, GTK_IMAGE (icon));
-	sync_label (view, NULL, label);
-	sync_load_status (view, NULL, hbox);
+	sync_label (embed, NULL, label);
+	sync_load_status (view, NULL, box);
+	sync_is_playing_audio (WEBKIT_WEB_VIEW (view), NULL, speaker_icon);
 
 	g_signal_connect_object (view, "notify::icon",
 				 G_CALLBACK (sync_icon), icon, 0);
-	g_signal_connect_object (view, "notify::embed-title",
+	g_signal_connect_object (embed, "notify::title",
 				 G_CALLBACK (sync_label), label, 0);
-	g_signal_connect_object (view, "notify::load-status",
-				 G_CALLBACK (sync_load_status), hbox, 0);
-
-	return hbox;
+	g_signal_connect_object (view, "load-changed",
+				 G_CALLBACK (load_changed_cb), box, 0);
+	g_signal_connect_object (view, "notify::is-playing-audio",
+				 G_CALLBACK (sync_is_playing_audio), speaker_icon, 0);
+	return box;
 }
 
 void
-ephy_notebook_set_show_tabs (EphyNotebook *nb,
-			     gboolean show_tabs)
+ephy_notebook_set_tabs_allowed (EphyNotebook *nb,
+				gboolean tabs_allowed)
 {
 	EphyNotebookPrivate *priv = nb->priv;
 
-	priv->show_tabs = show_tabs != FALSE;
+	priv->tabs_allowed = tabs_allowed != FALSE;
 
 	update_tabs_visibility (nb, FALSE);
+
+	g_object_notify (G_OBJECT (nb), "tabs-allowed");
 }
 
 static int
@@ -673,6 +705,10 @@ ephy_notebook_insert_page (GtkNotebook *gnotebook,
 
 	gtk_notebook_set_tab_reorderable (gnotebook, tab_widget, TRUE);
         gtk_notebook_set_tab_detachable (gnotebook, tab_widget, TRUE);
+	gtk_container_child_set (GTK_CONTAINER (gnotebook),
+				 GTK_WIDGET (tab_widget),
+				 "tab-expand", TRUE,
+				 NULL);
 
 	return position;
 }
@@ -692,6 +728,11 @@ ephy_notebook_add_tab (EphyNotebook *notebook,
 					     NULL,
 					     position);
 
+	gtk_container_child_set (GTK_CONTAINER (notebook),
+				 GTK_WIDGET (embed),
+				 "tab-expand", TRUE,
+				 NULL);
+
 	if (jump_to)
 	{
 		gtk_notebook_set_current_page (gnotebook, position);
@@ -739,11 +780,12 @@ ephy_notebook_remove (GtkContainer *container,
 	GtkNotebook *gnotebook = GTK_NOTEBOOK (container);
 	EphyNotebook *notebook = EPHY_NOTEBOOK (container);
 	EphyNotebookPrivate *priv = notebook->priv;
-	GtkWidget *tab_label, *tab_label_label, *tab_label_icon;
+	GtkWidget *tab_label, *tab_label_label, *tab_label_icon, *tab_label_speaker_icon;
 	int position, curr;
 	EphyWebView *view;
 
-	g_assert (EPHY_IS_EMBED (tab_widget));
+	if (!EPHY_IS_EMBED (tab_widget))
+		return;
 
 	/* Remove the page from the focused pages list */
 	priv->focused_pages =  g_list_remove (priv->focused_pages, tab_widget);
@@ -760,17 +802,83 @@ ephy_notebook_remove (GtkContainer *container,
 	tab_label = gtk_notebook_get_tab_label (gnotebook, tab_widget);
 	tab_label_icon = g_object_get_data (G_OBJECT (tab_label), "icon");
 	tab_label_label = g_object_get_data (G_OBJECT (tab_label), "label");
+	tab_label_speaker_icon = g_object_get_data (G_OBJECT (tab_label), "speaker-icon");
 
 	view = ephy_embed_get_web_view (EPHY_EMBED (tab_widget));
 
 	g_signal_handlers_disconnect_by_func
 		(view, G_CALLBACK (sync_icon), tab_label_icon);
 	g_signal_handlers_disconnect_by_func
-		(view, G_CALLBACK (sync_label), tab_label_label);
+		(tab_widget, G_CALLBACK (sync_label), tab_label_label);
 	g_signal_handlers_disconnect_by_func
 	  (view, G_CALLBACK (sync_load_status), tab_label);
+	g_signal_handlers_disconnect_by_func
+		(view, G_CALLBACK (sync_is_playing_audio), tab_label_speaker_icon);
 
 	GTK_CONTAINER_CLASS (ephy_notebook_parent_class)->remove (container, tab_widget);
 
 	update_tabs_visibility (notebook, FALSE);
 }
+
+/**
+ * ephy_notebook_next_page:
+ * @notebook: an #EphyNotebook
+ * 
+ * Advances to the next page in the @notebook. Note that unlike
+ * gtk_notebook_next_page() this method will wrap around if
+ * #GtkSettings:gtk-keynav-wrap-around is set.
+ **/
+void
+ephy_notebook_next_page (EphyNotebook *notebook)
+{
+	gint current_page, n_pages;
+
+	g_return_if_fail (EPHY_IS_NOTEBOOK (notebook));
+
+	current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+	n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+
+	if (current_page < n_pages - 1)
+		gtk_notebook_next_page (GTK_NOTEBOOK (notebook));
+	else {
+		gboolean  wrap_around;
+		
+		g_object_get (gtk_widget_get_settings (GTK_WIDGET (notebook)),
+			      "gtk-keynav-wrap-around", &wrap_around,
+			      NULL);
+		
+		if (wrap_around)
+			gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
+	}
+}
+
+/**
+ * ephy_notebook_prev_page:
+ * @notebook: an #EphyNotebook
+ *
+ * Advances to the previous page in the @notebook. Note that unlike
+ * gtk_notebook_next_page() this method will wrap around if
+ * #GtkSettings:gtk-keynav-wrap-around is set.
+ **/
+void
+ephy_notebook_prev_page (EphyNotebook *notebook)
+{
+	gint current_page;
+
+	g_return_if_fail (EPHY_IS_NOTEBOOK (notebook));
+
+	current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+
+	if (current_page > 0)
+		gtk_notebook_prev_page (GTK_NOTEBOOK (notebook));
+	else {
+		gboolean  wrap_around;
+		
+		g_object_get (gtk_widget_get_settings (GTK_WIDGET (notebook)),
+			      "gtk-keynav-wrap-around", &wrap_around,
+			      NULL);
+		
+		if (wrap_around)
+			gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), -1);
+	}
+}
