@@ -113,7 +113,6 @@ session_delete (EphySession *session,
 	g_object_unref (file);
 }
 
-#ifdef HAVE_WEBKIT2
 static void
 load_changed_cb (WebKitWebView *view,
 		 WebKitLoadEvent load_event,
@@ -122,22 +121,6 @@ load_changed_cb (WebKitWebView *view,
 	if (!ephy_web_view_load_failed (EPHY_WEB_VIEW (view)))
 		ephy_session_save (session, SESSION_STATE);
 }
-#else
-static void
-load_status_notify_cb (EphyWebView *view,
-		       GParamSpec *pspec,
-		       EphySession *session)
-{
-	WebKitLoadStatus status = webkit_web_view_get_load_status (WEBKIT_WEB_VIEW (view));
-
-	/* We won't know the URL we are loading in PROVISIONAL because
-	   of bug #593149, but save session anyway */
-	if (status == WEBKIT_LOAD_PROVISIONAL ||
-	    status == WEBKIT_LOAD_COMMITTED || 
-	    status == WEBKIT_LOAD_FINISHED)
-		ephy_session_save (session, SESSION_STATE);
-}
-#endif
 
 static gpointer *
 parent_location_new (EphyNotebook *notebook)
@@ -205,9 +188,6 @@ closed_tab_new (GQueue *closed_tabs,
 	ClosedTab *sibling_tab;
 
 	tab->url = g_strdup (address);
-#ifndef HAVE_WEBKIT2
-	tab->bflist = g_list_copy_deep (bflist, (GCopyFunc)webkit_web_history_item_copy, NULL);
-#endif
 	tab->position = position;
 
 	sibling_tab = find_tab_with_notebook (closed_tabs, parent_notebook);
@@ -251,14 +231,8 @@ ephy_session_undo_close_tab (EphySession *session)
 	EphySessionPrivate *priv;
 	EphyEmbed *embed, *new_tab;
 	ClosedTab *tab;
-#ifndef HAVE_WEBKIT2
-	WebKitWebBackForwardList *dest;
-	GList *i;
-#endif
-	EphyNewTabFlags flags = EPHY_NEW_TAB_OPEN_PAGE
-		| EPHY_NEW_TAB_PRESENT_WINDOW
-		| EPHY_NEW_TAB_JUMP
-		| EPHY_NEW_TAB_DONT_COPY_HISTORY;
+	EphyWindow *window;
+	EphyNewTabFlags flags = EPHY_NEW_TAB_JUMP;
 
 	g_return_if_fail (EPHY_IS_SESSION (session));
 
@@ -271,10 +245,6 @@ ephy_session_undo_close_tab (EphySession *session)
 	LOG ("UNDO CLOSE TAB: %s", tab->url);
 	if (*tab->parent_location != NULL)
 	{
-		GtkWidget *window;
-
-		flags |= EPHY_NEW_TAB_IN_EXISTING_WINDOW;
-
 		if (tab->position > 0)
 		{
 			/* Append in the n-th position. */
@@ -289,18 +259,19 @@ ephy_session_undo_close_tab (EphySession *session)
 			flags |= EPHY_NEW_TAB_FIRST;
 		}
 
-		window = gtk_widget_get_toplevel (GTK_WIDGET (*tab->parent_location));
+		window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (*tab->parent_location)));
 		new_tab = ephy_shell_new_tab (ephy_shell_get_default (),
-					      EPHY_WINDOW (window), embed, tab->url,
+					      window, embed,
 					      flags);
 		post_restore_cleanup (priv->closed_tabs, tab, FALSE);
 	}
 	else
 	{
 		EphyNotebook *notebook;
-		flags |=  EPHY_NEW_TAB_IN_NEW_WINDOW;
+
+		window = ephy_window_new ();
 		new_tab = ephy_shell_new_tab (ephy_shell_get_default (),
-					      NULL, NULL, tab->url, flags);
+					      window, NULL, flags);
 
 		/* FIXME: This makes the assumption that the notebook
 		   is the parent of the returned EphyEmbed. */
@@ -309,18 +280,10 @@ ephy_session_undo_close_tab (EphySession *session)
 		post_restore_cleanup (priv->closed_tabs, tab, TRUE);
 	}
 
-	/* This is deficient: we need to recreate the whole
-	 * BackForward list. Also, WebKit2 doesn't have this API. */
-#ifndef HAVE_WEBKIT2
-	dest = webkit_web_view_get_back_forward_list (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (new_tab));
-	for (i = tab->bflist; i; i = i->next)
-	{
-		LOG ("ADDING TO BF: %s",
-		     webkit_web_history_item_get_title ((WebKitWebHistoryItem*) i->data));
-		webkit_web_back_forward_list_add_item (dest,
-						       webkit_web_history_item_copy ((WebKitWebHistoryItem*) i->data));
-	}
-#endif
+	ephy_web_view_load_url (ephy_embed_get_web_view (new_tab), tab->url);
+	gtk_widget_grab_focus (GTK_WIDGET (new_tab));
+	gtk_window_present (GTK_WINDOW (window));
+
 	closed_tab_free (tab);
 
 	if (g_queue_is_empty (priv->closed_tabs))
@@ -336,11 +299,7 @@ ephy_session_tab_closed (EphySession *session,
 	EphySessionPrivate *priv = session->priv;
 	EphyWebView *view;
 	const char *address;
-#ifdef HAVE_WEBKIT2
 	WebKitBackForwardList *source;
-#else
-	WebKitWebBackForwardList *source;
-#endif
 	ClosedTab *tab;
 	GList *items = NULL;
 
@@ -348,11 +307,7 @@ ephy_session_tab_closed (EphySession *session,
 	address = ephy_web_view_get_address (view);
 
 	source = webkit_web_view_get_back_forward_list (WEBKIT_WEB_VIEW (view));
-#ifdef HAVE_WEBKIT2
 	items = webkit_back_forward_list_get_back_list_with_limit (source, EPHY_WEBKIT_BACK_FORWARD_LIMIT);
-#else
-	items = webkit_web_back_forward_list_get_back_list_with_limit (source, EPHY_WEBKIT_BACK_FORWARD_LIMIT);
-#endif
 	if (items == NULL && g_strcmp0 (address, "ephy-about:overview") == 0)
 		return;
 
@@ -395,13 +350,8 @@ notebook_page_added_cb (GtkWidget *notebook,
 			guint position,
 			EphySession *session)
 {
-#ifdef HAVE_WEBKIT2
 	g_signal_connect (ephy_embed_get_web_view (embed), "load-changed",
 			  G_CALLBACK (load_changed_cb), session);
-#else
-	g_signal_connect (ephy_embed_get_web_view (embed), "notify::load-status",
-			  G_CALLBACK (load_status_notify_cb), session);
-#endif
 }
 
 static void
@@ -412,15 +362,10 @@ notebook_page_removed_cb (GtkWidget *notebook,
 {
 	ephy_session_save (session, SESSION_STATE);
 
-#ifdef HAVE_WEBKIT2
 	g_signal_handlers_disconnect_by_func
 		(ephy_embed_get_web_view (embed), G_CALLBACK (load_changed_cb),
 		 session);
-#else
-	g_signal_handlers_disconnect_by_func
-		(ephy_embed_get_web_view (embed), G_CALLBACK (load_status_notify_cb),
-		 session);
-#endif
+
 	ephy_session_tab_closed (session, EPHY_NOTEBOOK (notebook), embed, position);
 }
 
@@ -442,14 +387,17 @@ session_maybe_open_window (EphySession *session,
 	/* FIXME: maybe just check for normal windows? */
 	if (ephy_shell_get_n_windows (shell) == 0)
 	{
-		ephy_shell_new_tab_full (shell,
-					 NULL /* window */, NULL /* tab */,
-					 NULL /* NetworkRequest */,
-					 EPHY_NEW_TAB_IN_NEW_WINDOW |
-					 EPHY_NEW_TAB_HOME_PAGE,
-					 EPHY_WEB_VIEW_CHROME_ALL,
-					 FALSE /* is popup? */,
-					 user_time);
+		EphyWindow *window = ephy_window_new ();
+		EphyEmbed *embed;
+
+		embed = ephy_shell_new_tab_full (shell,
+						 NULL /* title */,
+						 NULL /* related view */,
+						 window, NULL /* tab */,
+						 0,
+						 user_time);
+		ephy_web_view_load_homepage (ephy_embed_get_web_view (embed));
+		ephy_window_activate_location (window);
 	}
 }
 
@@ -625,12 +573,19 @@ session_tab_new (EphyEmbed *embed)
 	{
 		session_tab->url = g_strconcat ("about", address + EPHY_ABOUT_SCHEME_LEN, NULL);
 	}
+	else if (g_str_equal (address, "about:blank"))
+	{
+		/* EphyWebView address is NULL between load_uri() and WEBKIT_LOAD_STARTED,
+		 * but WebKitWebView knows the pending API request URL, so use that instead of about:blank.
+		 */
+		session_tab->url = g_strdup (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (web_view)));
+	}
 	else
 	{
 		session_tab->url = g_strdup (address);
 	}
 
-	session_tab->title = g_strdup (ephy_web_view_get_title (web_view));
+	session_tab->title = g_strdup (ephy_embed_get_title (embed));
 	session_tab->loading = ephy_web_view_is_loading (web_view) && !ephy_embed_has_load_pending (embed);
 
 	return session_tab;
@@ -963,10 +918,11 @@ confirm_before_recover (EphyWindow *window, const char *url, const char *title)
 {
 	EphyEmbed *embed;
 
-	embed = ephy_shell_new_tab (ephy_shell_get_default (),
-				    window, NULL, NULL,
-				    EPHY_NEW_TAB_IN_EXISTING_WINDOW |
-				    EPHY_NEW_TAB_APPEND_LAST);
+	embed = ephy_shell_new_tab_full (ephy_shell_get_default (),
+					 title, NULL,
+					 window, NULL,
+					 EPHY_NEW_TAB_APPEND_LAST,
+					 0);
 
 	ephy_web_view_load_error_page (ephy_embed_get_web_view (embed), url,
 			               EPHY_WEB_VIEW_ERROR_PAGE_CRASH, NULL);
@@ -1113,25 +1069,25 @@ session_parse_embed (SessionParserContext *context,
 		delay_loading = g_settings_get_boolean (EPHY_SETTINGS_MAIN,
 							EPHY_PREFS_RESTORE_SESSION_DELAYING_LOADS);
 
-		flags = EPHY_NEW_TAB_IN_EXISTING_WINDOW;
-		flags |= EPHY_NEW_TAB_APPEND_LAST;
+		flags = EPHY_NEW_TAB_APPEND_LAST;
 
+		embed = ephy_shell_new_tab_full (ephy_shell_get_default (),
+						 title, NULL,
+						 context->window, NULL, flags,
+						 0);
+
+		web_view = ephy_embed_get_web_view (embed);
 		if (delay_loading)
 		{
-			flags |= EPHY_NEW_TAB_DELAYED_OPEN_PAGE;
+			WebKitURIRequest *request = webkit_uri_request_new (url);
+
+			ephy_embed_set_delayed_load_request (embed, request);
+			ephy_web_view_set_placeholder (web_view, url, title);
+			g_object_unref (request);
 		}
 		else
 		{
-			flags |= EPHY_NEW_TAB_OPEN_PAGE;
-		}
-
-		embed = ephy_shell_new_tab (ephy_shell_get_default (),
-					    context->window, NULL, url, flags);
-
-		if (delay_loading)
-		{
-			web_view = ephy_embed_get_web_view (embed);
-			ephy_web_view_set_placeholder (web_view, url, title);
+			ephy_web_view_load_url (web_view, url);
 		}
 	}
 	else if (was_loading && url != NULL)
@@ -1365,7 +1321,10 @@ ephy_session_load_from_stream (EphySession *session,
 	session->priv->dont_save = TRUE;
 
 	task = g_task_new (session, cancellable, callback, user_data);
-	g_task_set_priority (task, G_PRIORITY_HIGH);
+	/* Use a priority lower than drawing events (HIGH_IDLE + 20) to make sure
+	 * the main window is shown as soon as possible at startup
+	 */
+	g_task_set_priority (task, G_PRIORITY_HIGH_IDLE + 30);
 
 	context = session_parser_context_new (session, user_time);
 	parser = g_markup_parse_context_new (&session_parser, 0, context, (GDestroyNotify)session_parser_context_free);
@@ -1507,7 +1466,10 @@ ephy_session_load (EphySession *session,
 	g_application_hold (G_APPLICATION (ephy_shell_get_default ()));
 
 	task = g_task_new (session, cancellable, callback, user_data);
-	g_task_set_priority (task, G_PRIORITY_HIGH);
+	/* Use a priority lower than drawing events (HIGH_IDLE + 20) to make sure
+	 * the main window is shown as soon as possible at startup
+	 */
+	g_task_set_priority (task, G_PRIORITY_HIGH_IDLE + 30);
 
 	save_to_file = get_session_file (filename);
 	data = load_async_data_new (user_time);
@@ -1586,7 +1548,7 @@ ephy_session_resume (EphySession *session,
 	EphyPrefsRestoreSessionPolicy policy;
 	EphyShell *shell;
 
-	LOG ("ephy_session_autoresume");
+	LOG ("ephy_session_resume");
 
 	task = g_task_new (session, cancellable, callback, user_data);
 
@@ -1615,7 +1577,7 @@ ephy_session_resume (EphySession *session,
 		return;
 	}
 
-	g_task_return_boolean (task, FALSE);
+	g_task_return_boolean (task, TRUE);
 	g_object_unref (task);
 }
 
