@@ -23,8 +23,8 @@
 #include "config.h"
 #include "prefs-dialog.h"
 
+#include "ephy-bookmarks.h"
 #include "ephy-debug.h"
-#include "ephy-dialog.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-shell.h"
@@ -38,7 +38,9 @@
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
-#include "pdm-dialog.h"
+#include "clear-data-dialog.h"
+#include "cookies-dialog.h"
+#include "passwords-dialog.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -57,17 +59,47 @@ enum
 	COL_LANG_CODE
 };
 
-#define EPHY_PREFS_DIALOG_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_PREFS_DIALOG, PrefsDialogPrivate))
-
 struct PrefsDialogPrivate
 {
+	/* general */
+	GtkWidget *download_button_hbox;
+	GtkWidget *download_button_label;
+	GtkWidget *automatic_downloads_checkbutton;
+	GtkWidget *search_engine_combo;
+	GtkWidget *popups_allow_checkbutton;
+	GtkWidget *adblock_allow_checkbutton;
+	GtkWidget *enable_plugins_checkbutton;
+
+	/* fonts */
+	GtkWidget *use_gnome_fonts_checkbutton;
+	GtkWidget *custom_fonts_table;
+	GtkWidget *sans_fontbutton;
+	GtkWidget *serif_fontbutton;
+	GtkWidget *mono_fontbutton;
+	GtkWidget *css_checkbox;
+	GtkWidget *css_edit_button;
+
+	/* privacy */
+	GtkWidget *always;
+	GtkWidget *no_third_party;
+	GtkWidget *never;
+	GtkWidget *remember_passwords_checkbutton;
+	GtkWidget *do_not_track_checkbutton;
+	GtkWidget *clear_personal_data_button;
+
+	/* language */
+	GtkWidget *default_encoding_combo;
 	GtkTreeView *lang_treeview;
-	GtkTreeModel *lang_model;
-	EphyDialog *add_lang_dialog;
 	GtkWidget *lang_add_button;
 	GtkWidget *lang_remove_button;
 	GtkWidget *lang_up_button;
 	GtkWidget *lang_down_button;
+	GtkWidget *enable_spell_checking_checkbutton;
+
+	GtkDialog *add_lang_dialog;
+	GtkTreeView *add_lang_treeview;
+	GtkTreeModel *lang_model;
+
 	GHashTable *iso_639_table;
 	GHashTable *iso_3166_table;
 };
@@ -78,7 +110,15 @@ enum {
 	NUM_COLS
 };
 
-G_DEFINE_TYPE (PrefsDialog, prefs_dialog, EPHY_TYPE_DIALOG)
+enum
+{
+	SEARCH_ENGINE_COL_NAME,
+	SEARCH_ENGINE_COL_STOCK_URL,
+	SEARCH_ENGINE_COL_URL,
+	SEARCH_ENGINE_NUM_COLS
+};
+
+G_DEFINE_TYPE_WITH_PRIVATE (PrefsDialog, prefs_dialog, GTK_TYPE_DIALOG)
 
 static void
 prefs_dialog_finalize (GObject *object)
@@ -88,11 +128,10 @@ prefs_dialog_finalize (GObject *object)
 
 	if (priv->add_lang_dialog != NULL)
 	{
-		EphyDialog **add_lang_dialog = &priv->add_lang_dialog;
+		GtkDialog **add_lang_dialog = &priv->add_lang_dialog;
 
-		g_object_remove_weak_pointer
-				(G_OBJECT (priv->add_lang_dialog),
-				 (gpointer *) add_lang_dialog);
+		g_object_remove_weak_pointer (G_OBJECT (priv->add_lang_dialog),
+					      (gpointer *) add_lang_dialog);
 		g_object_unref (priv->add_lang_dialog);
 	}
 
@@ -102,14 +141,81 @@ prefs_dialog_finalize (GObject *object)
 	G_OBJECT_CLASS (prefs_dialog_parent_class)->finalize (object);
 }
 
+static void
+on_manage_cookies_button_clicked (GtkWidget *button,
+				  PrefsDialog *dialog)
+{
+	CookiesDialog *cookies_dialog;
+
+	cookies_dialog = g_object_new (EPHY_TYPE_COOKIES_DIALOG,
+				       "use-header-bar", TRUE,
+				       NULL);
+	gtk_window_set_transient_for (GTK_WINDOW (cookies_dialog), GTK_WINDOW (dialog));
+	gtk_window_set_modal (GTK_WINDOW (cookies_dialog), TRUE);
+	gtk_window_present (GTK_WINDOW (cookies_dialog));
+}
+
+static void
+on_manage_passwords_button_clicked (GtkWidget *button,
+				    PrefsDialog *dialog)
+{
+	PasswordsDialog *passwords_dialog;
+
+	passwords_dialog = g_object_new (EPHY_TYPE_PASSWORDS_DIALOG,
+					 "use-header-bar", TRUE,
+					 NULL);
+	gtk_window_set_transient_for (GTK_WINDOW (passwords_dialog), GTK_WINDOW (dialog));
+	gtk_window_set_modal (GTK_WINDOW (passwords_dialog), TRUE);
+	gtk_window_present (GTK_WINDOW (passwords_dialog));
+}
+
 static void
 prefs_dialog_class_init (PrefsDialogClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	object_class->finalize = prefs_dialog_finalize;
 
-	g_type_class_add_private (object_class, sizeof(PrefsDialogPrivate));
+	gtk_widget_class_set_template_from_resource (widget_class,
+	                                             "/org/gnome/epiphany/prefs-dialog.ui");
+	/* general */
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, automatic_downloads_checkbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, search_engine_combo);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, popups_allow_checkbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, adblock_allow_checkbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, enable_plugins_checkbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, download_button_hbox);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, download_button_label);
+
+	/* fonts */
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, use_gnome_fonts_checkbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, custom_fonts_table);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, sans_fontbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, serif_fontbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, mono_fontbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, css_checkbox);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, css_edit_button);
+
+	/* privacy */
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, always);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, no_third_party);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, never);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, remember_passwords_checkbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, do_not_track_checkbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, clear_personal_data_button);
+
+	/* language */
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, default_encoding_combo);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, lang_treeview);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, lang_add_button);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, lang_remove_button);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, lang_up_button);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, lang_down_button);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, enable_spell_checking_checkbutton);
+
+	gtk_widget_class_bind_template_callback (widget_class, on_manage_cookies_button_clicked);
+	gtk_widget_class_bind_template_callback (widget_class, on_manage_passwords_button_clicked);
 }
 
 static void
@@ -151,6 +257,7 @@ combo_get_mapping (GValue *value,
 		if (g_strcmp0 (item_name, settings_name) == 0)
 		{
 			g_value_set_int (value, i);
+			g_free (item_name);
 			break;
 		}
 
@@ -191,7 +298,7 @@ combo_set_mapping (const GValue *value,
 }
 
 static void
-create_node_combo (EphyDialog *dialog,
+create_node_combo (PrefsDialog *dialog,
 		   EphyEncodings *encodings,
 		   const char *default_value)
 {
@@ -212,8 +319,7 @@ create_node_combo (EphyDialog *dialog,
 	}
 	g_free (code);
 
-	combo = GTK_COMBO_BOX (ephy_dialog_get_control (dialog,
-							"default_encoding_combo"));
+	combo = GTK_COMBO_BOX (dialog->priv->default_encoding_combo);
 
 	store = gtk_list_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_STRING);
 	all_encodings = ephy_encodings_get_all (encodings);
@@ -318,6 +424,10 @@ language_editor_update_pref (PrefsDialog *pd)
 		g_settings_set (EPHY_SETTINGS_WEB,
 				EPHY_PREFS_WEB_LANGUAGE,
 				"as", &builder);
+	} else {
+		g_settings_set (EPHY_SETTINGS_WEB,
+				EPHY_PREFS_WEB_LANGUAGE,
+				"as", NULL);
 	}
 }
 
@@ -354,13 +464,10 @@ language_editor_update_buttons (PrefsDialog *dialog)
 
 static void
 add_lang_dialog_selection_changed (GtkTreeSelection *selection,
-				   EphyDialog *dialog)
+				   GtkWidget *button)
 {
-	GtkWidget *button;
 	int n_selected;
 
-	button = ephy_dialog_get_control (dialog, "add_button");
-
 	n_selected = gtk_tree_selection_count_selected_rows (selection);
 	gtk_widget_set_sensitive (button, n_selected > 0);
 }
@@ -370,8 +477,7 @@ add_lang_dialog_response_cb (GtkWidget *widget,
 			     int response,
 			     PrefsDialog *pd)
 {
-	EphyDialog *dialog = pd->priv->add_lang_dialog;
-	GtkTreeView *treeview;
+	GtkDialog *dialog = pd->priv->add_lang_dialog;
 	GtkTreeModel *model;
 	GtkTreeSelection *selection;
 	GtkTreeIter iter;
@@ -381,9 +487,7 @@ add_lang_dialog_response_cb (GtkWidget *widget,
 
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
-		treeview = GTK_TREE_VIEW (ephy_dialog_get_control
-				(dialog, "languages_treeview"));
-		selection = gtk_tree_view_get_selection (treeview);
+		selection = gtk_tree_view_get_selection (pd->priv->add_lang_treeview);
 
 		rows = gtk_tree_selection_get_selected_rows (selection, &model);
 
@@ -394,18 +498,18 @@ add_lang_dialog_response_cb (GtkWidget *widget,
 			if (gtk_tree_model_get_iter (model, &iter, path))
 			{
 				char *code, *desc;
-				
+
 				gtk_tree_model_get (model, &iter,
 						    COL_LANG_NAME, &desc,
 						    COL_LANG_CODE, &code,
 						    -1);
 
 				language_editor_add (pd, code, desc);
-		
+
 				g_free (desc);
 				g_free (code);
 			}
-		}			
+		}
 
 		g_list_foreach (rows, (GFunc) gtk_tree_path_free, NULL);
 		g_list_free (rows);
@@ -414,7 +518,7 @@ add_lang_dialog_response_cb (GtkWidget *widget,
 		language_editor_update_buttons (pd);
 	}
 
-	g_object_unref (dialog);
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 static char *
@@ -497,11 +601,11 @@ add_system_language_entry (GtkListStore *store)
 	g_free (text);
 }
 
-static EphyDialog *
-setup_add_language_dialog (PrefsDialog *pd)
+static GtkDialog *
+setup_add_language_dialog (PrefsDialog *dialog)
 {
-	EphyDialog *dialog;
-	GtkWidget *window, *parent;
+	GtkWidget *ad;
+	GtkWidget *add_button;
 	GtkListStore *store;
 	GtkTreeModel *sortmodel;
 	GtkTreeView *treeview;
@@ -510,19 +614,13 @@ setup_add_language_dialog (PrefsDialog *pd)
 	GtkTreeSelection *selection;
 	GtkTreeIter iter;
 	int i;
+	GtkBuilder *builder;
 
-	parent = ephy_dialog_get_control (EPHY_DIALOG (pd), "prefs_dialog");
-
-	dialog =  EPHY_DIALOG (g_object_new (EPHY_TYPE_DIALOG,
-					     "parent-window", parent,
-					     "default-width", 260,
-					     "default-height", 230,
-					     NULL));
-
-	ephy_dialog_construct (dialog, 
-			       "/org/gnome/epiphany/prefs-dialog.ui",
-			       "add_language_dialog",
-			       NULL);
+	builder = gtk_builder_new_from_resource ("/org/gnome/epiphany/prefs-lang-dialog.ui");
+	ad = GTK_WIDGET (gtk_builder_get_object (builder, "add_language_dialog"));
+	add_button = GTK_WIDGET (gtk_builder_get_object (builder, "add_button"));
+	treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder, "languages_treeview"));
+	dialog->priv->add_lang_treeview = treeview;
 
 	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 
@@ -531,8 +629,8 @@ setup_add_language_dialog (PrefsDialog *pd)
 		const char *code = languages[i];
 		char *name;
 
-		name = get_name_for_lang_code (pd, code);
-		
+		name = get_name_for_lang_code (dialog, code);
+
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter,
 				    COL_LANG_NAME, name,
@@ -547,14 +645,9 @@ setup_add_language_dialog (PrefsDialog *pd)
 	gtk_tree_sortable_set_sort_column_id
 		(GTK_TREE_SORTABLE (sortmodel), COL_LANG_NAME, GTK_SORT_ASCENDING);
 
-	ephy_dialog_get_controls (dialog,
-				  "languages_treeview", &treeview,
-				  "add_language_dialog", &window,
-				  NULL);
-
-	gtk_window_group_add_window (gtk_window_get_group (GTK_WINDOW (parent)),
-				     GTK_WINDOW (window));
-	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
+	gtk_window_group_add_window (gtk_window_get_group (GTK_WINDOW (dialog)),
+				     GTK_WINDOW (ad));
+	gtk_window_set_modal (GTK_WINDOW (ad), TRUE);
 
 	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview), FALSE);
 
@@ -575,18 +668,18 @@ setup_add_language_dialog (PrefsDialog *pd)
 
 	selection = gtk_tree_view_get_selection (treeview);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
-	
-	add_lang_dialog_selection_changed (GTK_TREE_SELECTION (selection), dialog);
-	g_signal_connect (selection, "changed",
-			  G_CALLBACK (add_lang_dialog_selection_changed), dialog);
 
-	g_signal_connect (window, "response",
-			  G_CALLBACK (add_lang_dialog_response_cb), pd);
+	add_lang_dialog_selection_changed (GTK_TREE_SELECTION (selection), add_button);
+	g_signal_connect (selection, "changed",
+			  G_CALLBACK (add_lang_dialog_selection_changed), add_button);
+
+	g_signal_connect (ad, "response",
+			  G_CALLBACK (add_lang_dialog_response_cb), dialog);
 
 	g_object_unref (store);
 	g_object_unref (sortmodel);
 
-	return dialog;
+	return GTK_DIALOG (ad);
 }
 
 static void
@@ -595,9 +688,10 @@ language_editor_add_button_clicked_cb (GtkWidget *button,
 {
 	if (pd->priv->add_lang_dialog == NULL)
 	{
-		EphyDialog **add_lang_dialog;
+		GtkDialog **add_lang_dialog;
 
 		pd->priv->add_lang_dialog = setup_add_language_dialog (pd);
+		gtk_window_set_transient_for (GTK_WINDOW (pd->priv->add_lang_dialog), GTK_WINDOW (pd));
 
 		add_lang_dialog = &pd->priv->add_lang_dialog;
 
@@ -606,7 +700,7 @@ language_editor_add_button_clicked_cb (GtkWidget *button,
 			(gpointer *) add_lang_dialog);
 	}
 
-	ephy_dialog_show (pd->priv->add_lang_dialog);
+	gtk_window_present (GTK_WINDOW (pd->priv->add_lang_dialog));
 }
 
 static void
@@ -706,9 +800,9 @@ language_editor_selection_changed_cb (GtkTreeSelection *selection,
 }
 
 static void
-create_language_section (EphyDialog *dialog)
+create_language_section (PrefsDialog *dialog)
 {
-	PrefsDialog *pd = EPHY_PREFS_DIALOG (dialog);
+	PrefsDialogPrivate *priv = dialog->priv;
 	GtkListStore *store;
 	GtkTreeView *treeview;
 	GtkCellRenderer *renderer;
@@ -717,37 +811,28 @@ create_language_section (EphyDialog *dialog)
 	char **list = NULL;
 	int i;
 
-	pd->priv->iso_639_table = ephy_langs_iso_639_table ();
-	pd->priv->iso_3166_table = ephy_langs_iso_3166_table ();
+	priv->iso_639_table = ephy_langs_iso_639_table ();
+	priv->iso_3166_table = ephy_langs_iso_3166_table ();
 
-	ephy_dialog_get_controls
-		(dialog,
-		 "lang_treeview", &treeview,
-		 "lang_add_button", &pd->priv->lang_add_button,
-		 "lang_remove_button", &pd->priv->lang_remove_button,
-		 "lang_up_button", &pd->priv->lang_up_button,
-		 "lang_down_button", &pd->priv->lang_down_button,
-		 NULL);
-
-	g_signal_connect (pd->priv->lang_add_button, "clicked",
+	g_signal_connect (priv->lang_add_button, "clicked",
 			  G_CALLBACK (language_editor_add_button_clicked_cb), dialog);
-	g_signal_connect (pd->priv->lang_remove_button, "clicked",
+	g_signal_connect (priv->lang_remove_button, "clicked",
 			  G_CALLBACK (language_editor_remove_button_clicked_cb), dialog);
-	g_signal_connect (pd->priv->lang_up_button, "clicked",
+	g_signal_connect (priv->lang_up_button, "clicked",
 			  G_CALLBACK (language_editor_up_button_clicked_cb), dialog);
-	g_signal_connect (pd->priv->lang_down_button, "clicked",
+	g_signal_connect (priv->lang_down_button, "clicked",
 			  G_CALLBACK (language_editor_down_button_clicked_cb), dialog);
 
 	/* setup the languages treeview */
-	pd->priv->lang_treeview = treeview;
+	treeview = priv->lang_treeview;
 
 	gtk_tree_view_set_reorderable (treeview, TRUE);
 	gtk_tree_view_set_headers_visible (treeview, FALSE);
 
 	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 
-	pd->priv->lang_model = GTK_TREE_MODEL (store);
-	gtk_tree_view_set_model (treeview, pd->priv->lang_model);
+	priv->lang_model = GTK_TREE_MODEL (store);
+	gtk_tree_view_set_model (treeview, priv->lang_model);
 
 	renderer = gtk_cell_renderer_text_new ();
 
@@ -765,9 +850,9 @@ create_language_section (EphyDialog *dialog)
 
 	/* Connect treeview signals */
 	g_signal_connect (G_OBJECT (treeview), "drag_end",
-			  G_CALLBACK (language_editor_treeview_drag_end_cb), pd);
+			  G_CALLBACK (language_editor_treeview_drag_end_cb), dialog);
 	g_signal_connect (G_OBJECT (selection), "changed",
-			  G_CALLBACK (language_editor_selection_changed_cb), pd);
+			  G_CALLBACK (language_editor_selection_changed_cb), dialog);
 
 	list = g_settings_get_strv (EPHY_SETTINGS_WEB,
 				    EPHY_PREFS_WEB_LANGUAGE);
@@ -785,32 +870,32 @@ create_language_section (EphyDialog *dialog)
 		{
 			char *text;
 
-			text = get_name_for_lang_code (pd, code);
-			language_editor_add (pd, code, text);
+			text = get_name_for_lang_code (dialog, code);
+			language_editor_add (dialog, code, text);
 			g_free (text);
 		}
 	}
 	g_object_unref (store);
 
-	language_editor_update_buttons (pd);
+	language_editor_update_buttons (dialog);
 	g_strfreev (list);
 
 	/* Lockdown if key is not writable */
 	g_settings_bind_writable (EPHY_SETTINGS_WEB,
 				  EPHY_PREFS_WEB_LANGUAGE,
-				  pd->priv->lang_add_button, "sensitive", FALSE);
+				  priv->lang_add_button, "sensitive", FALSE);
 	g_settings_bind_writable (EPHY_SETTINGS_WEB,
 				  EPHY_PREFS_WEB_LANGUAGE,
-				  pd->priv->lang_remove_button, "sensitive", FALSE);
+				  priv->lang_remove_button, "sensitive", FALSE);
 	g_settings_bind_writable (EPHY_SETTINGS_WEB,
 				  EPHY_PREFS_WEB_LANGUAGE,
-				  pd->priv->lang_up_button, "sensitive", FALSE);
+				  priv->lang_up_button, "sensitive", FALSE);
 	g_settings_bind_writable (EPHY_SETTINGS_WEB,
 				  EPHY_PREFS_WEB_LANGUAGE,
-				  pd->priv->lang_down_button, "sensitive", FALSE);
+				  priv->lang_down_button, "sensitive", FALSE);
 	g_settings_bind_writable (EPHY_SETTINGS_WEB,
 				  EPHY_PREFS_WEB_LANGUAGE,
-				  pd->priv->lang_treeview, "sensitive", FALSE);
+				  priv->lang_treeview, "sensitive", FALSE);
 }
 
 static void
@@ -827,22 +912,16 @@ download_path_changed_cb (GtkFileChooser *button)
 }
 
 static void
-create_download_path_button (EphyDialog *dialog)
+create_download_path_button (PrefsDialog *dialog)
 {
-	GtkWidget *parent, *hbox, *label, *button;
+	GtkWidget *button;
 	EphyFileChooser *fc;
 	char *dir;
 
 	dir = ephy_file_get_downloads_dir ();
 
-	ephy_dialog_get_controls (dialog,
-				  "download_button_hbox", &hbox,
-				  "download_button_label", &label,
-				  "prefs_dialog", &parent,
-				  NULL);
-
 	fc = ephy_file_chooser_new (_("Select a Directory"),
-				    parent,
+				    GTK_WIDGET (dialog),
 				    GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
 				    EPHY_FILE_FILTER_NONE);
 
@@ -856,8 +935,8 @@ create_download_path_button (EphyDialog *dialog)
 						 DOWNLOAD_BUTTON_WIDTH);
 	g_signal_connect (button, "selection-changed",
 			  G_CALLBACK (download_path_changed_cb), dialog);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
-	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (dialog->priv->download_button_label), button);
+	gtk_box_pack_start (GTK_BOX (dialog->priv->download_button_hbox), button, TRUE, TRUE, 0);
 	gtk_widget_show (button);
 
 	g_settings_bind_writable (EPHY_SETTINGS_STATE,
@@ -869,27 +948,24 @@ create_download_path_button (EphyDialog *dialog)
 static void
 prefs_dialog_response_cb (GtkDialog *widget,
 			  int response,
-			  EphyDialog *dialog)
+			  GtkDialog *dialog)
 {
-	if (response == GTK_RESPONSE_HELP)
-	{
-		ephy_gui_help (GTK_WIDGET (widget), "pref");
-		return;
-	}
-		
-	g_object_unref (dialog);
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 static void
-clear_cache_button_clicked_cb (GtkWidget *button,
-			       PrefsDialog *dialog)
+clear_personal_data_button_clicked_cb (GtkWidget *button,
+				       PrefsDialog *dialog)
 {
-	GtkWidget *parent;
+	ClearDataDialog *clear_dialog;
 
-	parent = ephy_dialog_get_control (EPHY_DIALOG (dialog),
-					  "prefs_dialog");
-	pdm_dialog_show_clear_all_dialog (EPHY_DIALOG (dialog), parent,
-					  CLEAR_ALL_CACHE);
+	clear_dialog = g_object_new (EPHY_TYPE_CLEAR_DATA_DIALOG,
+				     "use-header-bar", TRUE,
+				     NULL);
+	clear_data_dialog_set_flags (clear_dialog, CLEAR_DATA_CACHE);
+	gtk_window_set_transient_for (GTK_WINDOW (clear_dialog), GTK_WINDOW (dialog));
+	gtk_window_set_modal (GTK_WINDOW (clear_dialog), TRUE);
+	gtk_window_present (GTK_WINDOW (clear_dialog));
 }
 
 static gboolean
@@ -903,6 +979,9 @@ cookies_get_mapping (GValue *value,
 	setting = g_variant_get_string (variant, NULL);
 	name = gtk_buildable_get_name (GTK_BUILDABLE (user_data));
 
+	if (g_strcmp0 (name, "no_third_party") == 0)
+		name = "no-third-party";
+
 	/* If the button name matches the setting, it should be active. */
 	if (g_strcmp0 (name, setting) == 0)
 		g_value_set_boolean (value, TRUE);
@@ -923,150 +1002,338 @@ cookies_set_mapping (const GValue *value,
 		return NULL;
 
 	name = gtk_buildable_get_name (GTK_BUILDABLE (user_data));
-	variant = g_variant_new_string (name);
+	if (g_strcmp0 (name, "no_third_party") == 0)
+		variant = g_variant_new_string ("no-third-party");
+	else
+		variant = g_variant_new_string (name);
 
 	return variant;
 }
 
-typedef struct
+static void
+search_engine_combo_add_default_engines (GtkListStore *store)
 {
-	char *obj;
-	char *prop;
-	char *schema;
-	char *key;
-	GSettingsBindFlags flags;
-	GSettingsBindGetMapping get_mapping;
-	GSettingsBindSetMapping set_mapping;
-} PrefsDialogPreference;
+	int i;
+	const char *default_engines[][3] = { /* Search engine option in the preferences dialog */
+					     { N_("DuckDuckGo"),
+					       "https://duckduckgo.com/?q=%s&t=epiphany",
+					       /* For the preferences dialog. Must exactly match the URL
+					        * you chose in the gschema, but with & instead of &amp;
+					        * If the match is not exact, there will be a spurious, ugly
+					        * entry in the preferences combo, so please test this. */
+					       N_("https://duckduckgo.com/?q=%s&t=epiphany")},
+					     /* Search engine option in the preferences dialog */
+					     { N_("Google"),
+					       "https://google.com/search?q=%s",
+					       /* For the preferences dialog. Consider a regional variant, like google.co.uk */
+					       N_("https://google.com/search?q=%s")},
+					     /* Search engine option in the preferences dialog */
+					     { N_("Bing"),
+					       "https://www.bing.com/search?q=%s",
+					       /* For the preferences dialog. Consider a regional variant, like uk.bing.com */
+					       N_("https://www.bing.com/search?q=%s")} };
 
-static const PrefsDialogPreference preferences[] =
-{
-	{ "automatic_downloads_checkbutton", "active",
-	  EPHY_PREFS_SCHEMA, EPHY_PREFS_AUTO_DOWNLOADS,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-	{ "remember_passwords_checkbutton", "active",
-	  EPHY_PREFS_SCHEMA, EPHY_PREFS_REMEMBER_PASSWORDS,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-	{ "do_not_track_checkbutton", "active",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_DO_NOT_TRACK,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-
-	{ "disk_cache_spinbutton", "value",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_CACHE_SIZE,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-	{ "use_gnome_fonts_checkbutton", "active",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_USE_GNOME_FONTS,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-	{ "popups_allow_checkbutton", "active",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_POPUPS,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-	{ "adblock_allow_checkbutton", "active",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_ADBLOCK,
-	  G_SETTINGS_BIND_INVERT_BOOLEAN, NULL, NULL },
-	{ "enable_plugins_checkbutton", "active",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_PLUGINS,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-	{ "css_checkbox", "active",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_USER_CSS,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-	{ "css_edit_button", "sensitive",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_USER_CSS,
-	  G_SETTINGS_BIND_GET, NULL, NULL },
-	{ "enable_spell_checking_checkbutton", "active",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-
-	/* Font buttons */
-	{ "custom_fonts_table", "sensitive",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_USE_GNOME_FONTS,
-	  G_SETTINGS_BIND_GET | G_SETTINGS_BIND_INVERT_BOOLEAN, NULL, NULL },
-
-	{ "sans_fontbutton", "font-name",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_SANS_SERIF_FONT,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-	{ "serif_fontbutton", "font-name",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_SERIF_FONT,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-	{ "mono_fontbutton", "font-name",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_MONOSPACE_FONT,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-
-	/* Has mapping */
-	{ "always", "active",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_COOKIES_POLICY,
-	  G_SETTINGS_BIND_DEFAULT, cookies_get_mapping, cookies_set_mapping },
-	{ "no-third-party", "active",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_COOKIES_POLICY,
-	  G_SETTINGS_BIND_DEFAULT, cookies_get_mapping, cookies_set_mapping },
-	{ "never", "active",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_COOKIES_POLICY,
-	  G_SETTINGS_BIND_DEFAULT, cookies_get_mapping, cookies_set_mapping },
-};
+	for (i = 0; i < G_N_ELEMENTS (default_engines); ++i)
+	{
+		gtk_list_store_insert_with_values (store, NULL, -1,
+						   SEARCH_ENGINE_COL_NAME,
+						   _(default_engines[i][0]),
+						   SEARCH_ENGINE_COL_STOCK_URL,
+						   default_engines[i][1],
+						   SEARCH_ENGINE_COL_URL,
+						   _(default_engines[i][2]),
+						   -1);
+	}
+}
 
 static void
-prefs_dialog_init (PrefsDialog *pd)
+search_engine_combo_add_smart_bookmarks (GtkListStore *store)
 {
-	EphyDialog *dialog = EPHY_DIALOG (pd);
-	EphyEncodings *encodings;
-	GtkWidget *window;
-	GtkWidget *clear_cache_button;
-	GtkWidget *css_checkbox, *css_edit_button;
 	int i;
+	EphyBookmarks *bookmarks;
+	EphyNode *smart_bookmarks_parent_node;
+	GPtrArray *smart_bookmarks;
 
-	pd->priv = EPHY_PREFS_DIALOG_GET_PRIVATE (pd);
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
+	smart_bookmarks_parent_node = ephy_bookmarks_get_smart_bookmarks (bookmarks);
+	smart_bookmarks = ephy_node_get_children (smart_bookmarks_parent_node);
 
-	ephy_dialog_construct (dialog,
-			       "/org/gnome/epiphany/prefs-dialog.ui",
-			       "prefs_dialog",
-			       NULL);
+	if (!smart_bookmarks)
+		return;
 
-	for (i = 0; i < G_N_ELEMENTS (preferences); i++)
+	for (i = 0; i < smart_bookmarks->len; ++i)
 	{
-		PrefsDialogPreference pref;
-		GtkWidget *widget;
-		GSettings *settings;
+		EphyNode *bookmark;
+		const char *bookmark_name;
+		const char *bookmark_url;
 
-		pref = preferences[i];
-		settings = ephy_settings_get (pref.schema);
-		widget = ephy_dialog_get_control (dialog, pref.obj);
+		bookmark = g_ptr_array_index (smart_bookmarks, i);
+		bookmark_name = ephy_node_get_property_string (bookmark, EPHY_NODE_BMK_PROP_TITLE);
+		bookmark_url = ephy_node_get_property_string (bookmark, EPHY_NODE_BMK_PROP_LOCATION);
 
-		if (pref.set_mapping != NULL || pref.get_mapping != NULL)
-			g_settings_bind_with_mapping (settings, pref.key,
-						      widget, pref.prop,
-						      pref.flags,
-						      pref.get_mapping,
-						      pref.set_mapping,
-						      widget, NULL);
-		else
-			g_settings_bind (settings, pref.key,
-					 widget, pref.prop,
-					 pref.flags);
+		if (!bookmark_name || !bookmark_url)
+			continue;
+
+		if (strcmp (bookmark_name, DEFAULT_SMART_BOOKMARK_TEXT) == 0)
+			continue;
+
+		gtk_list_store_insert_with_values (store, NULL, -1,
+						   SEARCH_ENGINE_COL_NAME, bookmark_name,
+						   SEARCH_ENGINE_COL_STOCK_URL, bookmark_url,
+						   SEARCH_ENGINE_COL_URL, bookmark_url,
+						   -1);
+	}
+}
+
+/* Has the user manually set the engine to something not in the combo?
+ * If so, add that URL as an extra item in the combo. */
+static void
+search_engine_combo_add_current_engine (GtkListStore *store)
+{
+	GtkTreeIter iter;
+	char *original_url;
+	gboolean in_combo = FALSE;
+	gboolean has_next = FALSE;
+
+	original_url = g_settings_get_string (EPHY_SETTINGS_MAIN,
+					      EPHY_PREFS_KEYWORD_SEARCH_URL);
+	if (!original_url)
+		return;
+
+	has_next = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter);
+
+	while (!in_combo && has_next)
+	{
+		char *stock_url, *url;
+
+		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
+				    SEARCH_ENGINE_COL_STOCK_URL, &stock_url,
+				    SEARCH_ENGINE_COL_URL, &url, -1);
+
+		if (strcmp (original_url, stock_url) == 0 ||
+		    strcmp (original_url, url) == 0)
+			in_combo = TRUE;
+
+		g_free (stock_url);
+		g_free (url);
+		has_next = gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter);
 	}
 
-	ephy_dialog_get_controls (dialog,
-				  "prefs_dialog", &window,
-				  "css_checkbox", &css_checkbox,
-				  "css_edit_button", &css_edit_button,
-				  "clear_cache_button", &clear_cache_button,
-				  NULL);
+	if (!in_combo)
+		gtk_list_store_insert_with_values (store, NULL, -1,
+						   SEARCH_ENGINE_COL_NAME, original_url,
+						   SEARCH_ENGINE_COL_STOCK_URL, original_url,
+						   SEARCH_ENGINE_COL_URL, original_url,
+						   -1);
 
-	ephy_gui_ensure_window_group (GTK_WINDOW (window));
+	g_free (original_url);
+}
 
-	g_signal_connect (window, "response",
-			  G_CALLBACK (prefs_dialog_response_cb), dialog);
+static void
+create_search_engine_combo (GtkComboBox *combo)
+{
+	GtkCellRenderer *renderer;
+	GtkListStore *store;
 
-	g_signal_connect (css_edit_button, "clicked",
-			  G_CALLBACK (css_edit_button_clicked_cb), dialog);
+	store = GTK_LIST_STORE (gtk_combo_box_get_model (combo));
 
-	g_signal_connect (clear_cache_button, "clicked",
-			  G_CALLBACK (clear_cache_button_clicked_cb), dialog);
+	search_engine_combo_add_default_engines (store);
+	search_engine_combo_add_smart_bookmarks (store);
+	search_engine_combo_add_current_engine (store);
 
-	encodings = EPHY_ENCODINGS (ephy_embed_shell_get_encodings
-					(EPHY_EMBED_SHELL (ephy_shell_get_default ())));
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), SEARCH_ENGINE_COL_NAME,
+					      GTK_SORT_ASCENDING);
+	gtk_combo_box_set_model (combo, GTK_TREE_MODEL (store));
+
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer,
+					"text", SEARCH_ENGINE_COL_NAME,
+					NULL);
+
+	g_settings_bind_with_mapping (EPHY_SETTINGS_MAIN,
+				      EPHY_PREFS_KEYWORD_SEARCH_URL,
+				      combo, "active",
+				      G_SETTINGS_BIND_DEFAULT,
+				      combo_get_mapping,
+				      combo_set_mapping,
+				      combo,
+				      NULL);
+}
+
+static void
+setup_general_page (PrefsDialog *dialog)
+{
+	PrefsDialogPrivate *priv = dialog->priv;
+	GSettings *settings;
+	GSettings *web_settings;
+
+	settings = ephy_settings_get (EPHY_PREFS_SCHEMA);
+	web_settings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
+
+	g_settings_bind (settings,
+			 EPHY_PREFS_AUTO_DOWNLOADS,
+			 priv->automatic_downloads_checkbutton,
+			 "active",
+			 G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_ENABLE_POPUPS,
+			 priv->popups_allow_checkbutton,
+			 "active",
+			 G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_ENABLE_ADBLOCK,
+			 priv->adblock_allow_checkbutton,
+			 "active",
+			 G_SETTINGS_BIND_INVERT_BOOLEAN);
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_ENABLE_PLUGINS,
+			 priv->enable_plugins_checkbutton,
+			 "active",
+			 G_SETTINGS_BIND_DEFAULT);
+
+	create_download_path_button (dialog);
+	create_search_engine_combo (GTK_COMBO_BOX (dialog->priv->search_engine_combo));
+}
+
+static void
+setup_fonts_page (PrefsDialog *dialog)
+{
+	PrefsDialogPrivate *priv = dialog->priv;
+	GSettings *web_settings;
+
+	web_settings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
+
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_USE_GNOME_FONTS,
+			 priv->use_gnome_fonts_checkbutton,
+			 "active",
+			 G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_USE_GNOME_FONTS,
+			 priv->custom_fonts_table,
+			 "sensitive",
+			 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_INVERT_BOOLEAN);
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_SANS_SERIF_FONT,
+			 priv->sans_fontbutton,
+			 "font-name",
+			 G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_SERIF_FONT,
+			 priv->serif_fontbutton,
+			 "font-name",
+			 G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_MONOSPACE_FONT,
+			 priv->mono_fontbutton,
+			 "font-name",
+			 G_SETTINGS_BIND_DEFAULT);
+
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_ENABLE_USER_CSS,
+			 priv->css_checkbox,
+			 "active",
+			 G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_ENABLE_USER_CSS,
+			 priv->css_edit_button,
+			 "sensitive",
+			 G_SETTINGS_BIND_GET);
+	g_signal_connect (priv->css_edit_button,
+			  "clicked",
+			  G_CALLBACK (css_edit_button_clicked_cb),
+			  dialog);
+}
+
+static void
+setup_privacy_page (PrefsDialog *dialog)
+{
+	PrefsDialogPrivate *priv = dialog->priv;
+	GSettings *settings;
+	GSettings *web_settings;
+
+	settings = ephy_settings_get (EPHY_PREFS_SCHEMA);
+	web_settings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
+
+	g_settings_bind_with_mapping (web_settings,
+				      EPHY_PREFS_WEB_COOKIES_POLICY,
+				      priv->always,
+				      "active",
+				      G_SETTINGS_BIND_DEFAULT,
+				      cookies_get_mapping,
+				      cookies_set_mapping,
+				      priv->always,
+				      NULL);
+	g_settings_bind_with_mapping (web_settings,
+				      EPHY_PREFS_WEB_COOKIES_POLICY,
+				      priv->no_third_party,
+				      "active",
+				      G_SETTINGS_BIND_DEFAULT,
+				      cookies_get_mapping,
+				      cookies_set_mapping,
+				      priv->no_third_party,
+				      NULL);
+	g_settings_bind_with_mapping (web_settings,
+				      EPHY_PREFS_WEB_COOKIES_POLICY,
+				      priv->never,
+				      "active",
+				      G_SETTINGS_BIND_DEFAULT,
+				      cookies_get_mapping,
+				      cookies_set_mapping,
+				      priv->never,
+				      NULL);
+	g_settings_bind (settings,
+			 EPHY_PREFS_REMEMBER_PASSWORDS,
+			 priv->remember_passwords_checkbutton,
+			 "active",
+			 G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_DO_NOT_TRACK,
+			 priv->do_not_track_checkbutton,
+			 "active",
+			 G_SETTINGS_BIND_DEFAULT);
+
+	g_signal_connect (priv->clear_personal_data_button,
+			  "clicked",
+			  G_CALLBACK (clear_personal_data_button_clicked_cb),
+			  dialog);
+}
+
+static void
+setup_language_page (PrefsDialog *dialog)
+{
+	PrefsDialogPrivate *priv = dialog->priv;
+	GSettings *web_settings;
+	EphyEncodings *encodings;
+
+	web_settings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
+
+	g_settings_bind (web_settings,
+			 EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING,
+			 priv->enable_spell_checking_checkbutton,
+			 "active",
+			 G_SETTINGS_BIND_DEFAULT);
+
+	encodings = EPHY_ENCODINGS (ephy_embed_shell_get_encodings (EPHY_EMBED_SHELL (ephy_shell_get_default ())));
 
 	create_node_combo (dialog, encodings, "ISO-8859-1");
 
-	create_language_section	(dialog);
+	create_language_section (dialog);
+}
 
-	create_download_path_button (dialog);
+static void
+prefs_dialog_init (PrefsDialog *dialog)
+{
+	dialog->priv = prefs_dialog_get_instance_private (dialog);
+	gtk_widget_init_template (GTK_WIDGET (dialog));
+
+	setup_general_page (dialog);
+	setup_fonts_page (dialog);
+	setup_privacy_page (dialog);
+	setup_language_page (dialog);
+
+	ephy_gui_ensure_window_group (GTK_WINDOW (dialog));
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (prefs_dialog_response_cb), dialog);
 }
