@@ -84,7 +84,7 @@ fixture_setup (Fixture *fixture, gconstpointer data)
   char *tmp_filename;
   char *dest_file;
 
-  tmp_filename = ephy_file_tmp_filename ("ephy-download-XXXXXX", NULL);
+  tmp_filename = ephy_file_tmp_filename (".ephy-download-XXXXXX", NULL);
   dest_file = g_build_filename (ephy_file_tmp_dir (), tmp_filename, NULL);
 
   fixture->source = get_uri_for_path ("/default");
@@ -141,8 +141,11 @@ test_ephy_download_new (Fixture *fixture, gconstpointer data)
 static void
 test_ephy_download_new_for_uri (Fixture *fixture, gconstpointer data)
 {
-  g_assert_cmpstr (fixture->source, ==,
-                   ephy_download_get_source_uri (fixture->download));
+  WebKitDownload *download = ephy_download_get_webkit_download (fixture->download);
+  WebKitURIRequest *request = webkit_download_get_request (download);
+
+  g_assert(request);
+  g_assert_cmpstr (fixture->source, ==, webkit_uri_request_get_uri (request));
 }
 
 static void
@@ -152,7 +155,6 @@ test_ephy_download_start (Fixture *fixture, gconstpointer data)
                     G_CALLBACK (completed_cb), fixture);
 
   ephy_download_set_action (fixture->download, EPHY_DOWNLOAD_ACTION_DO_NOTHING);
-  ephy_download_start (fixture->download);
   g_main_loop_run (fixture->loop);
 }
 
@@ -160,12 +162,12 @@ int
 main (int argc, char *argv[])
 {
   int ret;
+  GSList* uris;
   SoupServer *server;
 
   gtk_test_init (&argc, &argv);
 
   ephy_debug_init ();
-  ephy_embed_prefs_init ();
 
   if (!ephy_file_helpers_init (NULL,
                                EPHY_FILE_HELPERS_PRIVATE_PROFILE | EPHY_FILE_HELPERS_ENSURE_EXISTS,
@@ -175,12 +177,15 @@ main (int argc, char *argv[])
   }
 
   _ephy_shell_create_instance (EPHY_EMBED_SHELL_MODE_TEST);
+  g_application_register (G_APPLICATION (ephy_shell_get_default ()), NULL, NULL);
 
-  server = soup_server_new (SOUP_SERVER_PORT, 0, NULL);
-  soup_server_run_async (server);
-
-  base_uri = soup_uri_new ("http://127.0.0.1/");
-  soup_uri_set_port (base_uri, soup_server_get_port (server));
+  server = soup_server_new (NULL, NULL);
+  soup_server_listen_local (server, 0,
+                            SOUP_SERVER_LISTEN_IPV4_ONLY,
+                            NULL);
+  uris = soup_server_get_uris (server);
+  base_uri = (SoupURI*)uris->data;
+  g_slist_free (uris);
 
   soup_server_add_handler (server, NULL, server_callback, NULL, NULL);
 
